import re

from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


def _make_cron_re():
    r"""
    Make the regular expression that matches a crontab 'cron' line.

    Each field has a set of allowed values, and can then be in a range, and be
    listed with dashes.  A range can be stepped with the '/' modifier, and
    ranges can be in a list.  A field can also be '*', or '*' divided in steps.

    The best way to do this is to have a template for a single field that
    encapsulates the syntax of that field, regardless of what that field
    matches.  We then fill in the actual template's value with the pattern
    that matches that field.  Each field is named, so we can pull them out as
    a dictionary later.
    """
    range_ = r"{val}(?:-{val}(?:/\d+)?)?"
    template = r"(?P<{name}>" + r"(?:\*(?:/\d+)?|{r}(?:,{r})*)".format(r=range_) + r")\s+"
    return (
        r'^\s*' +
        template.format(name='minute', val=r'(?:\d|[012345]\d)') +
        template.format(name='hour', val=r'(?:\d|[01]\d|2[0123])') +
        template.format(name='day_of_month', val=r'(?:0?[1-9]|[12]\d|3[01])') +
        template.format(name='month', val=r'(?:0?[1-9]|1[012]|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)') +
        template.format(name='day_of_week', val=r'(?:[0-7]|mon|tue|wed|thur|fri|sat|sun)') +
        r'(?P<command>\S.*)$'
    )


class CrontabL(CommandParser):
    r"""
    Parses output of ``crontab -l`` command.

    Each row of the crontab is converted into a dictionary with keys for each field.
    For example one row would look like::

        {
          'minute': '*',
          'hour': '*',
          'day_of_month': '*',
          'month': '*',
          'day_of_week': '*',
          'command': '/usr/bin/keystone-manage token_flush > /dev/null 2>&1'
        }

    Crontab parses the line in the same way that cron(1) does.  Lines that
    are blank or start with a comment are ignored.  Environment lines of the
    form 'KEY = value' (with optional spacing around the equals sign) are
    stored in the 'environment' dictionary attribute by key.  Lines
    containing a valid crontab line, with five recogniseable time fields and
    a command, are stored in the `data` property and accessed through the
    pseudo-list interface and search method.  All other lines are stored in
    the `invalid_lines` property.

    Crontab recognises the extension of time signature 'nicknames', which
    take place of the first five parts of a standard crontab line:

        * `@reboot` : Run once after reboot.
        * `@yearly` : Run once a year, ie.  "0 0 1 1 \*".
        * `@annually` : Run once a year, ie.  "0 0 1 1 \*".
        * `@monthly` : Run once a month, ie. "0 0 1 \* \*".
        * `@weekly` : Run once a week, ie.  "0 0 \* \* 0".
        * `@daily` : Run once a day, ie.   "0 0 \* \* \*".
        * `@hourly` : Run once an hour, ie. "0 \* \* \* \*".

    The Crontab class recognises these nicknames.  In the case of the '@reboot'
    nickname, the row will not contain the 'minute', 'hour', 'day_of_month',
    'month', or 'day_of_week' keys, and instead will contain the key 'time' with
    the value '@reboot' (as well as the usual 'command' key).  All other keywords
    are translated directly into their five-part equivalent and parsed as a normal
    crontab line.

    Lines that can't be parsed because they don't contain at least six words or
    meet the criteria for environment line or time signature nicknames are stored
    in the ``invalid_lines`` property.

    Sample input looks like::

        # send mail to admin address.
        MAILTO=admin
        * * * * * /usr/bin/keystone-manage token_flush > /dev/null 2>&1
        33 0 * * * /bin/heat-manage purge_deleted -g days 7

    Examples:
        >>> crontab = shared[KeystoneCrontab]
        >>> crontab.search('keystone')
        [{'minute': '*', 'hour': '*', 'day_of_month': '*', 'month': '*', 'day_of_week': '*',
          'command': '/usr/bin/keystone-manage token_flush > /dev/null 2>&1'}]
        >>> [r['minute'] for r in crontab]
        ['*', '33']
        >>> crontab.invalid_lines
        []
        >>> len(crontab)  # Number of actual entries
        2
        >>> crontab[0]  # Individual entry access
        {'minute': '*', 'hour': '*', 'day_of_month': '*', 'month': '*', 'day_of_week': '*',
          'command': '/usr/bin/keystone-manage token_flush > /dev/null 2>&1'}
        >>> 'MAILTO' in crontab.environment  # Dictionary of environment settings
        True
        >>> crontab.environment['MAILTO']
        'admin'

    Attributes:

        data(list): List of parsed lines.  These can be accessed directly through
            the object as seen in the examples.
        environment(dict): A dictionary of environment declarations in the crontab.
        invalid_lines(list): Lines that could not be parsed as normal crontab
            entries.
    """
    def parse_content(self, content):
        self.data = []
        self.environment = {}
        self.invalid_lines = []
        # Crontabs can use 'nicknames' for common event frequencies:
        nicknames = {
            '@yearly': '0 0 1 1 *',
            '@annually': '0 0 1 1 *',
            '@monthly': '0 0 1 * *',
            '@weekly': '0 0 * * 0',
            '@daily': '0 0 * * *',
            '@hourly': '0 * * * *',
        }
        cron_re = re.compile(_make_cron_re(), flags=re.IGNORECASE)
        env_re = re.compile(r'^\s*(?P<key>\w+)\s*=\s*(?P<value>\S.*)$')
        for line in get_active_lines(content):
            if line.startswith('@'):
                # Reboot is 'special':
                if line.startswith('@reboot'):
                    parts = line.split(None, 2)
                    self.data.append({'time': '@reboot', 'command': parts[1]})
                    continue
                else:
                    parts = line.split(None, 2)
                    if parts[0] not in nicknames:
                        raise ParseException(
                            "{n} not recognised as a time specification 'nickname'".format(n=parts[0])
                        )
                    # Otherwise, put the time spec nickname translation into
                    # the line
                    line = line.replace(parts[0], nicknames[parts[0]])
                    # And then we fall through to the rest of the parsing.
            cron_match = cron_re.match(line)
            env_match = env_re.match(line)
            if cron_match:
                self.data.append(cron_match.groupdict())
            elif env_match:
                # Environment variable - capture in dictionary
                self.environment[env_match.group('key')] = env_match.group('value')
            else:
                self.invalid_lines.append(line)

    def __getitem__(self, idx):
        """
        Get the n'th item from the crontab table.

        Arguments:
            idx(int): The index of the crontab line, starting from 0.

        Returns:
            (dict): the number of items
        """
        return self.data[idx]

    def __iter__(self):
        """
        Iterates through the list of crontab entries.

        Yields:
            (dict): The next parsed crontab entry.
        """
        for row in self.data:
            yield row

    def search(self, filter_str):
        """list: Returns list of dicts for lines that have `filter_str` in
        the command."""
        return [r for r in self.data if filter_str in r['command']]


@parser(Specs.heat_crontab)
class HeatCrontab(CrontabL):
    """Parses output of the ``crontab -l -u heat`` command."""
    pass


@parser(Specs.keystone_crontab)
class KeystoneCrontab(CrontabL):
    """Parses output of the ``crontab -l -u keystone`` command."""
    pass


@parser(Specs.nova_crontab)
class NovaCrontab(CrontabL):
    """Parses output of the ``crontab -l -u nova`` command."""
    pass


@parser(Specs.root_crontab)
class RootCrontab(CrontabL):
    """Parses output of the ``crontab -l -u root`` command."""
    pass



import doctest
import pytest

from insights.parsers import rpm_pkgs
from insights.parsers.rpm_pkgs import RpmPkgsWritable
from insights.tests import context_wrap


DATASOURCE_OUTPUT = ["httpd-core|httpd-core-2.4.53-7.el9.x86_64|Red Hat, Inc."]

RPM_PKGS_WRITABLE_OUTPUT = [("httpd-core", "httpd-core-2.4.53-7.el9.x86_64", "Red Hat, Inc.")]


@pytest.mark.parametrize("parser, output", [
    (RpmPkgsWritable, RPM_PKGS_WRITABLE_OUTPUT)
])
def test_rpm_pkgs(parser, output):
    test = parser(context_wrap(DATASOURCE_OUTPUT))
    assert test.packages == output


def test_doc_examples():
    env = {
        "rpm_pkgs_writable": RpmPkgsWritable(context_wrap(DATASOURCE_OUTPUT))
    }
    failed, total = doctest.testmod(rpm_pkgs, globs=env)
    assert failed == 0



"""
GetconfPageSize - command ``/usr/sbin/getconf PAGE_SIZE``
=========================================================

This very simple parser returns the output of the ``getconf PAGE_SIZE`` command.

Examples:

    >>> pagesize_parsed.page_size
    4096
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.getconf_page_size)
class GetconfPageSize(CommandParser):
    """Class for parsing 'getconf PAGE_SIZE' command output

    Output: page_size

    Attributes:
        page_size (int): returns the page_size in bytes depending upon the architecture
    """

    def parse_content(self, content):
        if len(content) != 1:
            msg = "getconf PAGE_SIZE output contains multiple non-empty lines"
            raise ParseException(msg)
        raw = content[0].strip()
        self.page_size = int(raw)

    def __str__(self):
        return "<page_size: {}>".format(self.page_size)



from insights.parsers.crictl_ps import CrictlPs
from insights.tests import context_wrap
from insights.core.exceptions import ParseException, SkipComponent
import pytest


CRICTL_PS = """
CONTAINER           IMAGE                                                                                                                        CREATED              STATE               NAME                                          ATTEMPT             POD ID              POD
93b10093a8263       bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c                                                             About a minute ago   Running             oauth-apiserver                               185                 19d971fe5c478       apiserver-7cd97c59ff-dwckz
e34ce05ade472       2c96c7c72cf99490b4bdbb7389020b7e4b5bb7dc43ea9cadc4d5af43cb300b3f                                                             9 days ago           Running             guard                                         1                   c48cc19e5b0b1       etcd-guard-nah-4jnq5-master-v8z5h-0
471d75b135b5b       90e50eece96ef2a252b729a76a2ee3360d3623295cceb7d3e623b55cb7aef30a                                                             9 days ago           Running             etcd                                          39                  d2dd84f8db754       etcd-nah-4jnq5-master-v8z5h-0
""".strip()

CRICTL_PS_WITH_DIFFERENT_STATES = """
CONTAINER           IMAGE                                                                                                                        CREATED              STATE               NAME                                          ATTEMPT             POD ID              POD
93b10093a8263       bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c                                                             About a minute ago   Running             oauth-apiserver                               185                 19d971fe5c478       apiserver-7cd97c59ff-dwckz
e34ce05ade472       2c96c7c72cf99490b4bdbb7389020b7e4b5bb7dc43ea9cadc4d5af43cb300b3f                                                             2 hours ago          Exited              guard                                         1                   c48cc19e5b0b1       etcd-guard-nah-4jnq5-master-v8z5h-0
471d75b135b5b       90e50eece96ef2a252b729a76a2ee3360d3623295cceb7d3e623b55cb7aef30a                                                             1 day ago            ContainerCreating   etcd                                          39                  d2dd84f8db754       etcd-nah-4jnq5-master-v8z5h-0
abc123def456        sha256:1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef             3 weeks ago         Unknown              test-container                              5                   f1f2f3f4f5f6       test-pod-123
""".strip()

CRICTL_PS_EMPTY = """
CONTAINER           IMAGE                                                                                                                        CREATED              STATE               NAME                                          ATTEMPT             POD ID              POD
""".strip()

CRICTL_PS_INVALID_HEADER = """
INVALID HEADER
93b10093a8263       bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c                                                             About a minute ago   Running             oauth-apiserver                               185                 19d971fe5c478       apiserver-7cd97c59ff-dwckz
""".strip()


def test_crictl_ps():
    """Test basic crictl ps parsing with standard output"""
    result = CrictlPs(context_wrap(CRICTL_PS))

    # Test number of records
    assert len(result) == 3

    # Test first record
    first_record = result[0]
    assert first_record['container_id'] == '93b10093a8263'
    assert first_record['image'] == 'bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c'
    assert first_record['created'] == 'About a minute ago'
    assert first_record['state'] == 'Running'
    assert first_record['name'] == 'oauth-apiserver'
    assert first_record['attempt'] == '185'
    assert first_record['pod_id'] == '19d971fe5c478'
    assert first_record['pod'] == 'apiserver-7cd97c59ff-dwckz'

    # Test second record
    second_record = result[1]
    assert second_record['container_id'] == 'e34ce05ade472'
    assert second_record['image'] == '2c96c7c72cf99490b4bdbb7389020b7e4b5bb7dc43ea9cadc4d5af43cb300b3f'
    assert second_record['created'] == '9 days ago'
    assert second_record['state'] == 'Running'
    assert second_record['name'] == 'guard'
    assert second_record['attempt'] == '1'
    assert second_record['pod_id'] == 'c48cc19e5b0b1'
    assert second_record['pod'] == 'etcd-guard-nah-4jnq5-master-v8z5h-0'

    # Test third record
    third_record = result[2]
    assert third_record['container_id'] == '471d75b135b5b'
    assert third_record['image'] == '90e50eece96ef2a252b729a76a2ee3360d3623295cceb7d3e623b55cb7aef30a'
    assert third_record['created'] == '9 days ago'
    assert third_record['state'] == 'Running'
    assert third_record['name'] == 'etcd'
    assert third_record['attempt'] == '39'
    assert third_record['pod_id'] == 'd2dd84f8db754'
    assert third_record['pod'] == 'etcd-nah-4jnq5-master-v8z5h-0'


def test_crictl_ps_different_states():
    """Test crictl ps parsing with different container states"""
    result = CrictlPs(context_wrap(CRICTL_PS_WITH_DIFFERENT_STATES))

    # Test number of records
    assert len(result) == 4

    # Test different states
    states = [record['state'] for record in result]
    assert 'Running' in states
    assert 'Exited' in states
    assert 'ContainerCreating' in states
    assert 'Unknown' in states

    # Test different created formats
    created_values = [record['created'] for record in result]
    assert 'About a minute ago' in created_values
    assert '2 hours ago' in created_values
    assert '1 day ago' in created_values
    assert '3 weeks ago' in created_values

    # Test record with different image format (sha256:)
    sha256_record = result[3]
    assert sha256_record['container_id'] == 'abc123def456'
    assert sha256_record['image'] == 'sha256:1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef'
    assert sha256_record['state'] == 'Unknown'
    assert sha256_record['name'] == 'test-container'
    assert sha256_record['attempt'] == '5'
    assert sha256_record['pod_id'] == 'f1f2f3f4f5f6'
    assert sha256_record['pod'] == 'test-pod-123'


def test_crictl_ps_empty():
    """Test crictl ps parsing with empty data"""
    with pytest.raises(SkipComponent) as excinfo:
        CrictlPs(context_wrap(CRICTL_PS_EMPTY))

    assert "No container records found" in str(excinfo.value)


def test_crictl_ps_invalid_header():
    """Test crictl ps parsing with invalid header"""
    with pytest.raises(ParseException) as excinfo:
        CrictlPs(context_wrap(CRICTL_PS_INVALID_HEADER))

    assert "invalid content" in str(excinfo.value)


def test_crictl_ps_container_ids():
    """Test accessing container IDs"""
    result = CrictlPs(context_wrap(CRICTL_PS))

    # Test container_ids property
    expected_ids = ['93b10093a8263', 'e34ce05ade472', '471d75b135b5b']
    assert result.container_ids == expected_ids


def test_crictl_ps_indexing():
    """Test indexing and iteration"""
    result = CrictlPs(context_wrap(CRICTL_PS))

    # Test indexing
    assert result[0]['container_id'] == '93b10093a8263'
    assert result[1]['container_id'] == 'e34ce05ade472'
    assert result[2]['container_id'] == '471d75b135b5b'

    # Test length
    assert len(result) == 3

    # Test iteration
    container_ids = []
    for record in result:
        container_ids.append(record['container_id'])

    expected_ids = ['93b10093a8263', 'e34ce05ade472', '471d75b135b5b']
    assert container_ids == expected_ids


def test_crictl_ps_contains():
    """Test contains method"""
    result = CrictlPs(context_wrap(CRICTL_PS))

    # Test that records are in the data
    first_record = result[0]
    assert first_record in result

    # Test that a non-existent record is not in the data
    fake_record = {
        'container_id': 'fake', 'image': 'fake', 'created': 'fake',
        'state': 'fake', 'name': 'fake', 'attempt': 'fake',
        'pod_id': 'fake', 'pod': 'fake'
    }
    assert fake_record not in result


def test_crictl_ps_field_types():
    """Test that all required fields are present and have correct types"""
    result = CrictlPs(context_wrap(CRICTL_PS))

    required_fields = ['container_id', 'image', 'created', 'state', 'name', 'attempt', 'pod_id', 'pod']

    for record in result:
        # Check all required fields are present
        for field in required_fields:
            assert field in record
            assert isinstance(record[field], str)
            assert record[field] != ''  # Fields should not be empty


def test_crictl_ps_edge_cases():
    """Test edge cases in parsing"""
    # Test with minimal valid data
    minimal_data = """
CONTAINER           IMAGE                                                                                                                        CREATED              STATE               NAME                                          ATTEMPT             POD ID              POD
abc123             def456                                                                                                                      1 hour ago           Running             test                                         1                   pod123              test-pod
""".strip()

    result = CrictlPs(context_wrap(minimal_data))
    assert len(result) == 1

    record = result[0]
    assert record['container_id'] == 'abc123'
    assert record['image'] == 'def456'
    assert record['created'] == '1 hour ago'
    assert record['state'] == 'Running'
    assert record['name'] == 'test'
    assert record['attempt'] == '1'
    assert record['pod_id'] == 'pod123'
    assert record['pod'] == 'test-pod'


def test_crictl_ps_examples():
    """Test the examples from the docstring"""
    result = CrictlPs(context_wrap(CRICTL_PS))

    # Test the examples from the docstring
    assert len(result) == 3
    assert result[0]['container_id'] == '93b10093a8263'
    assert result[0]['image'] == 'bea2d277eb71530a376a68be9760260cedb59f2392bb6e7793b05d5350df8d4c'
    assert result[0]['created'] == 'About a minute ago'
    assert result[0]['state'] == 'Running'
    assert result[0]['name'] == 'oauth-apiserver'
    assert result[0]['attempt'] == '185'
    assert result[0]['pod_id'] == '19d971fe5c478'
    assert result[0]['pod'] == 'apiserver-7cd97c59ff-dwckz'



import pytest

from insights.core.exceptions import SkipComponent
from insights.specs.datasources.dev import physical_devices
from insights.combiners.virt_what import VirtWhat
from insights.core import dr
from insights.parsers.blkid import BlockIDInfo
from insights.parsers.virt_what import VirtWhat as VWP
from insights.tests import context_wrap


BLKID_INFO = """
/dev/sda1: UUID="3676157d-f2f5-465c-a4c3-fffffffff" TYPE="xfs"
/dev/sda2: UUID="UVTk76-UWOc-vk7s-galL-dxIP-fffffffff" TYPE="LVM2_member"
/dev/mapper/rhel_hp--dl160g8--3-root: UUID="11124c1d-990b-4277-9f74-fffffffff" TYPE="xfs"
/dev/mapper/rhel_hp--dl160g8--3-swap: UUID="c7c45f2d-1d1b-4cf0-9d51-fffffffff" TYPE="swap"
/dev/loop0: LABEL="Satellite-5.6.0 x86_64 Disc 0" TYPE="iso9660"
/dev/block/253:1: UUID="f8508c37-eeb1-4598-b084-fffffffff" TYPE="ext2"
/dev/sdb1: UUID="4676157d-f2f5-465c-a4c3-fffffffff" TYPE="xfs"
""".strip()

BLKID_INFO2 = """
/dev/vda1: UUID="3676157d-f2f5-465c-a4c3-fffffffff" TYPE="xfs"
/dev/vda2: UUID="UVTk76-UWOc-vk7s-galL-dxIP-fffffffff" TYPE="LVM2_member"
/dev/mapper/rhel_hp--dl160g8--3-root: UUID="11124c1d-990b-4277-9f74-fffffffff" TYPE="xfs"
/dev/mapper/rhel_hp--dl160g8--3-swap: UUID="c7c45f2d-1d1b-4cf0-9d51-fffffffff" TYPE="swap"
/dev/loop0: LABEL="Satellite-5.6.0 x86_64 Disc 0" TYPE="iso9660"
/dev/block/253:1: UUID="f8508c37-eeb1-4598-b084-fffffffff" TYPE="ext2"
/dev/vdb1: UUID="4676157d-f2f5-465c-a4c3-fffffffff" TYPE="xfs"
""".strip()

VW_OUT1 = """
kvm
""".strip()

# baremetal retuns blank
VW_OUT2 = """

""".strip()


def test_physical_devices():
    virt_what_info = VirtWhat(None, VWP(context_wrap(VW_OUT2)))
    blkid_info = BlockIDInfo(context_wrap(BLKID_INFO))

    broker = dr.Broker()
    broker[BlockIDInfo] = blkid_info
    broker[VirtWhat] = virt_what_info
    result = physical_devices(broker)
    assert result == ["/dev/sda", "/dev/sdb"]


def test_physical_devices_virt_env():
    virt_what_info = VirtWhat(None, VWP(context_wrap(VW_OUT1)))
    blkid_info = BlockIDInfo(context_wrap(BLKID_INFO))

    broker = dr.Broker()
    broker[BlockIDInfo] = blkid_info
    broker[VirtWhat] = virt_what_info
    with pytest.raises(SkipComponent):
        physical_devices(broker)


def test_physical_devices_virt_env2():
    virt_what_info = VirtWhat(None, VWP(context_wrap(VW_OUT1)))
    blkid_info = BlockIDInfo(context_wrap(BLKID_INFO2))

    broker = dr.Broker()
    broker[BlockIDInfo] = blkid_info
    broker[VirtWhat] = virt_what_info
    with pytest.raises(SkipComponent):
        physical_devices(broker)



from __future__ import print_function

import copy
import importlib
import inspect
import itertools
import json
import logging
import six
import six.moves

from collections import defaultdict
from functools import wraps
from operator import eq
from yaml import SafeLoader

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import insights

from insights import apply_filters
from insights.core import dr, filters, spec_factory, _PatchedSafeLoader  # noqa: F401
from insights.core.context import Context
from insights.core.plugins import make_none
from insights.specs import Specs


# Use yaml.SafeLoader when testing plugins, as the yaml.CSafeLoader
# does not work well with coverage test.
insights.core._PatchedSafeLoader = SafeLoader
# we intercept the add_filter call during integration testing so we can ensure
# that rules add filters to datasources that *should* be filterable
ADDED_FILTERS = defaultdict(set)
add_filter = filters.add_filter
find = spec_factory.find


def _intercept_add_filter(func):
    @wraps(func)
    def inner(component, pattern, max_match=filters.MAX_MATCH):
        ret = add_filter(component, pattern, max_match=max_match)
        calling_module = inspect.stack()[1][0].f_globals.get("__name__")
        ADDED_FILTERS[calling_module] |= set(
            r for r in dr.get_registry_points(component) if r.filterable
        )
        return ret

    return inner


def _intercept_find(func):
    @wraps(func)
    def inner(ds, pattern):
        ret = find(ds, pattern)
        calling_module = inspect.stack()[1][0].f_globals.get("__name__")
        ADDED_FILTERS[calling_module].add(ds)
        return ret

    return inner


filters.add_filter = _intercept_add_filter(filters.add_filter)
insights.add_filter = _intercept_add_filter(insights.add_filter)

spec_factory.find = _intercept_find(spec_factory.find)


logger = logging.getLogger(__name__)

ARCHIVE_GENERATORS = []
HEARTBEAT_ID = "99e26bb4823d770cc3c11437fe075d4d1a4db4c7500dad5707faed3b"
HEARTBEAT_NAME = "insights-heartbeat-9cd6f607-6b28-44ef-8481-62b0e7773614"

DEFAULT_RELEASE = "Red Hat Enterprise Linux Server release 7.2 (Maipo)"
DEFAULT_HOSTNAME = "hostname.example.com"

MAKE_NONE_RESULT = make_none()

_UNDEFINED = object()


def _beautify_deep_compare_diff(result, expected):
    if not (isinstance(result, dict) and isinstance(expected, dict)):
        return result

    if result.get('type') == 'skip':
        return result

    expected_keys = set(expected.keys())
    result_keys = set(result.keys())
    common_keys = set.intersection(result_keys, expected_keys)

    diff = []
    for k in result_keys - common_keys:
        diff.append('\tkey "{0}" not in Expected;'.format(k))
    for k in expected_keys - common_keys:
        diff.append('\tkey "{0}" not in Result;'.format(k))
    for k in common_keys:
        if not eq(result[k], expected[k]):
            diff.append(
                '\tkey "{0}" unequal values:\n\t\tExpected: {1}\n\t\tResult  : {2}'.format(
                    k, expected[k], result[k]
                )
            )
    if not diff:
        diff.append('\tUnrecognized unequal values in result layer one;')

    diff.append('Result: "{0}"'.format(result))
    return '\n' + '\n'.join(diff)


def deep_compare(result, expected):
    """
    Deep compare rule reducer results when testing.

    .. note::
        "[None, XX]" is a special format of the `expected` for this methoed to
        check the missing dependencies.
    """
    logger.debug("--Comparing-- (%s) %s to (%s) %s", type(result), result, type(expected), expected)

    missing = None
    if isinstance(expected, (tuple, list, set)) and len(expected) == 2 and expected[0] is None:
        expected, missing = expected

    # This case ensures that when rules return a make_none() response, all of the older
    # CI tests that are looking for None instead of make_none() will still pass
    if result is None or (isinstance(result, dict) and result.get("type") == "none"):
        assert expected is None or expected == MAKE_NONE_RESULT, result
        return

    if isinstance(result, dict) and expected is None:
        # checking the missing component (RHINRULE-283)
        if missing:
            assert "MISSING_REQUIREMENTS" == result['reason'], result['reason']
            for mis in [missing] if isinstance(missing, str) else missing:
                assert mis in result['details'], '"{0}" not in "{1}"'.format(mis, result['details'])
        assert result["type"] == "skip", result
        return

    assert eq(result, expected), _beautify_deep_compare_diff(result, expected)


def run_input_data(component, input_data, store_skips=False):
    broker = dr.Broker()
    broker.store_skips = store_skips
    for k, v in input_data.data.items():
        broker[k] = v

    graph = dr.get_dependency_graph(component)
    broker = dr.run(graph, broker=broker)
    for v in broker.tracebacks.values():
        logger.warning(v)
    return broker


COMPONENT_FILTERED_PARSERS = {
    'BootLoaderEntries': ['insights.specs.Specs.ls_lan_filtered'],
    'CloudInstance': ['insights.parsers.subscription_manager.SubscriptionManagerFacts'],
    'CloudProvider': [
        'insights.parsers.rhsm_conf.RHSMConf',
        'insights.parsers.subscription_manager.SubscriptionManagerFacts',
    ],
    'Grubby': ['insights.specs.Specs.ls_lan_filtered'],
    'GrubConf': ['insights.specs.Specs.ls_lan_filtered'],
    'IsUEFIBoot': ['insights.specs.Specs.ls_lan_filtered'],
    'OSRelease': ['insights.parsers.dmesg.DmesgLineList'],
    'Sap': ['insights.parsers.saphostctrl.SAPHostCtrlInstances'],
    'SELinux': ['insights.specs.Specs.ls_lan_filtered'],
}


def run_test(component, input_data, expected=_UNDEFINED, return_make_none=False, do_filter=True):
    """
    Arguments:
        component: The insights component need to test.
        input_data: The test data prepared for testing the component.
        expected: The expected result need to compare.
        return_make_none: Does it allow to return None?
        do_filter: Does need to check dependency spec filter warning?
            - it's not required to check the filters for sosreport
    """

    def get_filtered_specs(module):
        filtered = set()
        mods = dir(importlib.import_module(module))
        for comp, parsers in COMPONENT_FILTERED_PARSERS.items():
            if comp in mods:
                for parser in parsers:
                    if parser.split('.')[-1] not in mods:
                        # The parser is NOT imported again in the rule
                        parser = dr.get_component_by_name(parser)
                        filtered.update(dr.get_registry_points(parser))
        return filtered

    if do_filter and filters.ENABLED:
        mod = dr.get_module_name(component)
        sup_mod = '.'.join(mod.split('.')[:-1])
        rps = dr.get_registry_points(component)
        filtered = get_filtered_specs(mod)
        filterable = set(d for d in rps if dr.get_delegate(d).filterable) - filtered
        missing_filters = (
            filterable - ADDED_FILTERS.get(mod, set()) - ADDED_FILTERS.get(sup_mod, set())
        )
        if missing_filters:
            names = [dr.get_name(m) for m in missing_filters]
            msg = "%s must add filters to %s"
            raise Exception(msg % (mod, ", ".join(names)))

    broker = run_input_data(component, input_data)
    result = broker.get(component)
    if expected is not _UNDEFINED:
        deep_compare(result, expected)
    elif result == MAKE_NONE_RESULT and not return_make_none:
        # Convert make_none() result to None as default unless
        # make_none explicitly requested
        return None
    return result


def integrate(input_data, component):
    return run_test(component, input_data)


def context_wrap(
    lines,
    path="path",
    hostname=DEFAULT_HOSTNAME,
    release=DEFAULT_RELEASE,
    version="-1.-1",
    machine_id="machine_id",
    strip=True,
    split=True,
    filtered_spec=None,
    **kwargs
):
    if isinstance(lines, six.string_types):
        if strip:
            lines = lines.strip()
        if split:
            lines = lines.splitlines()

    if filtered_spec is not None and filtered_spec in filters.FILTERS:
        lines = [l for l in lines if any([f in l for f in filters.FILTERS[filtered_spec]])]

    return Context(
        content=lines,
        path=path,
        hostname=hostname,
        release=release,
        version=version.split("."),
        machine_id=machine_id,
        relative_path=path,
        **kwargs
    )


input_data_cache = {}


counter = itertools.count()


def create_metadata(system_id, product):
    ctx_metadata = {"system_id": system_id, "links": []}
    ctx_metadata["type"] = product.role
    ctx_metadata["product"] = product.__class__.__name__
    return json.dumps(ctx_metadata)


class InputData(object):
    """
    Helper class used with integrate. The role of this class is to represent
    data files sent to parsers and rules in a format similar to what lays on
    disk.

    Example Usage::

        input_data = InputData()
        input_data.add("messages", "this is some messages content")
        input_data.add("uname", "this is some uname content")

    If `release` is specified when InputData is constructed, it will be
    added to every *mock* record when added. This is useful for testing parsers
    that rely on context.release.

    If `path` is specified when calling the `add` method, the record will
    contain the specified value in the context.path field.  This is useful for
    testing pattern-like file parsers.
    """

    def __init__(self, name=None, hostname=None):
        cnt = input_data_cache.get(name, 0)
        self.name = "{0}-{1:0>5}".format(name, cnt)
        self.data = {}
        input_data_cache[name] = cnt + 1
        if hostname:
            self.add(Specs.hostname, hostname)

    def __setitem__(self, key, value):
        self.add(key, value)

    def __getitem__(self, key):
        return self.data[key]

    def _make_path(self):
        return str(next(counter)) + "BOGUS"

    def get(self, key, default):
        return self.data.get(key, default)

    def items(self):
        return self.data.items()

    def clone(self, name):
        the_clone = InputData(name)
        the_clone.data = {}
        for k, v in self.data.items():
            the_clone.data[k] = copy.deepcopy(v)
        return the_clone

    def add_component(self, comp, obj):
        """
        Allow adding arbitrary objects as components. This allows tests to mock
        components that have external dependencies so their dependents can be
        integration tested.
        """
        self.data[comp] = obj

    def add(self, spec, content, path=None, do_filter=True, **kwargs):
        if not path:  # path must change to allow parsers to fire
            path = self._make_path()
        if not path.startswith("/"):
            path = "/" + path

        if dr.get_delegate(spec).raw:
            content_iter = content
        else:
            if not isinstance(content, list):
                content_iter = [l.rstrip() for l in StringIO(content).readlines()]
            else:
                content_iter = content

            if do_filter:
                content_iter = list(apply_filters(spec, content_iter))

        content_provider = context_wrap(content_iter, path=path, split=False, **kwargs)
        if dr.get_delegate(spec).multi_output:
            if spec not in self.data:
                self.data[spec] = []
            self.data[spec].append(content_provider)
        else:
            self.data[spec] = content_provider

        return self

    def __repr__(self):
        if self.name:
            return "<InputData {name:%s}>" % (self.name)
        else:
            return super(InputData, self).__repr__()


# Helper constants when its necessary to test for a specific RHEL major version
# eg RHEL6, but the minor version isn't important
RHEL4 = "Red Hat Enterprise Linux AS release 4 (Nahant Update 9)"
RHEL5 = "Red Hat Enterprise Linux Server release 5.11 (Tikanga)"
RHEL6 = "Red Hat Enterprise Linux Server release 6.5 (Santiago)"
RHEL7 = "Red Hat Enterprise Linux Server release 7.0 (Maipo)"
RHEL8 = "Red Hat Enterprise Linux release 8.0 (Ootpa)"


def redhat_release(major, minor=""):
    """
    Helper function to construct a redhat-release string for a specific RHEL
    major and minor version.  Only constructs redhat-releases for RHEL major
    releases 4, 5, 6 & 7

    Arguments:
        major: RHEL major number. Accepts str, int or float (as major.minor)
        minor: RHEL minor number. Optional and accepts str or int

    For example, to construct a redhat-release for::

        RHEL4U9:  redhat_release('4.9') or (4.9) or (4, 9)
        RHEL5 GA: redhat_release('5')   or (5.0) or (5, 0) or (5)
        RHEL6.6:  redhat_release('6.6') or (6.6) or (6, 6)
        RHEL7.1:  redhat_release('7.1') or (7.1) or (7, 1)

    Limitation with float args: (x.10) will be parsed as minor = 1
    """
    if isinstance(major, str) and "." in major:
        major, minor = major.split(".")
    elif isinstance(major, float):
        major, minor = str(major).split(".")
    elif isinstance(major, int):
        major = str(major)
    if isinstance(minor, int):
        minor = str(minor)

    if major == "4":
        if minor:
            minor = "" if minor == "0" else " Update %s" % minor
        return "Red Hat Enterprise Linux AS release %s (Nahant%s)" % (major, minor)

    if major == "8":
        if not minor:
            minor = "0"
        return "Red Hat Enterprise Linux release %s.%s Beta (Ootpa)" % (major, minor)

    template = "Red Hat Enterprise Linux Server release %s%s (%s)"
    if major == "5":
        if minor:
            minor = "" if minor == "0" else "." + minor
        return template % (major, minor, "Tikanga")
    elif major == "6" or major == "7":
        if not minor:
            minor = "0"
        name = "Santiago" if major == "6" else "Maipo"
        return template % (major, "." + minor, name)
    else:
        raise Exception("invalid major version: %s" % major)


def archive_provider(component, test_func=deep_compare, stride=1):
    """
    Decorator used to register generator functions that yield InputData and
    expected response tuples.  These generators will be consumed by py.test
    such that:

    - Each InputData will be passed into an integrate() function
    - The result will be compared [1] against the expected value from the
      tuple.

    Parameters
    ----------
    component: (str)
        The component to be tested.
    test_func: function
        A custom comparison function with the parameters (result, expected).
        This will override the use of the compare() [1] function.
    stride: int
        yield every `stride` InputData object rather than the full set. This
        is used to provide a faster execution path in some test setups.

    [1] insights.tests.deep_compare()
    """

    def _wrap(func):
        @six.wraps(func)
        def __wrap(stride=stride):
            for input_data, expected in itertools.islice(func(), None, None, stride):
                yield component, test_func, input_data, expected

        __wrap.stride = stride
        ARCHIVE_GENERATORS.append(__wrap)
        return __wrap

    return _wrap



import doctest
import pytest

from insights.core.exceptions import ContentException, ParseException, SkipComponent
from insights.parsers import azure_instance
from insights.parsers.azure_instance import (
    AzureInstanceID,
    AzureInstancePlan,
    AzureInstanceType,
    AzurePublicIpv4Addresses,
)
from insights.tests import context_wrap

# For AzureInstanceID
AZURE_ID_1 = "f904ece8-c6c1-4b5c-881f-309b50f25e50"
AZURE_ID_2 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k

f904ece8-c6c1-4b5c-881f-309b50f25e50
""".strip()
AZURE_ID_AB_1 = '<?xml version="1.0" encoding="utf-8"?>'
AZURE_ID_AB_2 = 'internal server error '

# For AzureInstanceType
AZURE_TYPE_1 = "Standard_L32s"
AZURE_TYPE_2 = "Standard_NV48s_v3"
AZURE_TYPE_3 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
Standard_NV48s_v3
"""
AZURE_TYPE_DOC = "Standard_L64s_v2"
AZURE_TYPE_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
AZURE_TYPE_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
AZURE_TYPE_AB_3 = """
curl: (28) connect() timed out!
""".strip()
AZURE_TYPE_AB_4 = """
.micro
""".strip()
AZURE_TYPE_AB_5 = """
No module named insights.tools
""".strip()

# For AzureInstancePlan
AZURE_PLAN_1 = '{"name": "rhel7", "product": "rhel", "publisher": "Red Hat"}'
AZURE_PLAN_2 = '{"name": "", "product": "", "publisher": "Red Hat"}'
AZURE_PLAN_3 = '{"name": "", "product": "", "publisher": ""}'

AZURE_PLAN_4 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
{"name": "rhel7", "product": "rhel", "publisher": "Red Hat"}
"""

AZURE_PLAN_DOC = '{"name": "planName", "product": "planProduct", "publisher": "planPublisher"}'

AZURE_PLAN_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
AZURE_PLAN_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
AZURE_PLAN_AB_3 = """
curl: (28) connect() timed out!
""".strip()

# For AzurePublicIpv4Addresses and AzurePublicHostname
AZURE_LB_1 = """
{
  "loadbalancer": {
    "publicIpAddresses": [
      {
        "frontendIpAddress": "137.116.118.209",
        "privateIpAddress": "10.0.0.4"
      }
    ],
    "inboundRules": [],
    "outboundRules": []
  }
}
""".strip()

AZURE_LB_ERR1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
AZURE_LB_ERR2 = """
curl: (7) couldn't connect to host
""".strip()
AZURE_LB_ERR3 = """
curl: (28) connect() timed out!
""".strip()
AZURE_LB_ERR4 = "}}}"
AZURE_LB_ERR5 = """
{ "error": "No load balancer metadata is found. Please check if your VM is using any non-basic SKU load balancer and retry later." }
""".strip()
AZURE_LB_ERR6 = """
{
  "loadbalancer": {
    "publicIpAddresses": [
      {
        "frontendIpAddress": "",
        "privateIpAddress": "10.0.0.4"
      },
      {
        "privateIpAddress": "10.0.0.4"
      }
    ],
    "inboundRules": [],
    "outboundRules": []
  }
}
""".strip()


# Test AzureInstanceID
def test_azure_instance_id_ab_empty():
    with pytest.raises(SkipComponent):
        AzureInstanceID(context_wrap(''))


def test_azure_instance_id():
    azure = AzureInstanceID(context_wrap(AZURE_ID_1))
    assert azure.id == "f904ece8-c6c1-4b5c-881f-309b50f25e50"
    assert repr(azure) == "<instance_id: {0}>".format(azure.id)

    azure = AzureInstanceID(context_wrap(AZURE_ID_2))
    assert azure.id == "f904ece8-c6c1-4b5c-881f-309b50f25e50"
    assert repr(azure) == "<instance_id: {0}>".format(azure.id)


def test_azure_instance_id_ab():
    with pytest.raises(ParseException):
        AzureInstanceID(context_wrap(AZURE_ID_AB_1))

    with pytest.raises(ParseException):
        AzureInstanceID(context_wrap(AZURE_ID_AB_2))


# Test AzureInstanceType
def test_azure_instance_type_ab_other():
    with pytest.raises(SkipComponent):
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_1))

    with pytest.raises(SkipComponent):
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_2))

    with pytest.raises(SkipComponent):
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_3))

    with pytest.raises(ParseException) as pe:
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_4))
        assert 'Unrecognized type' in str(pe)

    with pytest.raises(ContentException) as pe:
        AzureInstanceType(context_wrap(AZURE_TYPE_AB_5))


def test_azure_instance_type_ab_empty():
    with pytest.raises(SkipComponent):
        AzureInstanceType(context_wrap(''))


def test_azure_instance_type():
    azure = AzureInstanceType(context_wrap(AZURE_TYPE_1))
    assert azure.type == "Standard"
    assert azure.size == "L32s"
    assert azure.version is None
    assert azure.raw == "Standard_L32s"
    assert repr(azure) == "<azure_type: {t}, size: {s}, version: {v},  raw: {r}>".format(
        t=azure.type, s=azure.size, v=azure.version, r=azure.raw
    )

    azure = AzureInstanceType(context_wrap(AZURE_TYPE_2))
    assert azure.type == "Standard"
    assert azure.size == "NV48s"
    assert azure.version == "v3"
    assert azure.raw == "Standard_NV48s_v3"
    assert "NV48s" in str(azure)


def test_azure_instance_type_stats():
    azure = AzureInstanceType(context_wrap(AZURE_TYPE_3))
    assert azure.type == "Standard"
    assert azure.size == "NV48s"
    assert azure.version == "v3"
    assert azure.raw == "Standard_NV48s_v3"


# Test AzureInstancePlan
def test_azure_instance_plan_ab_other():
    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(AZURE_PLAN_AB_1))

    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(AZURE_PLAN_AB_2))

    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(AZURE_PLAN_AB_3))

    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(''))

    with pytest.raises(ParseException):
        AzureInstancePlan(context_wrap(AZURE_PLAN_4))


def test_azure_instance_plan():
    azure = AzureInstancePlan(context_wrap(AZURE_PLAN_1))
    assert azure.name == "rhel7"
    assert azure.product == "rhel"
    assert azure.publisher == "Red Hat"
    assert azure.raw == '{"name": "rhel7", "product": "rhel", "publisher": "Red Hat"}'
    assert repr(azure) == "<azure_plan_name: {n}, product: {pr}, publisher: {pu}, raw: {r}".format(
        n=azure.name, pr=azure.product, pu=azure.publisher, r=azure.raw
    )

    azure = AzureInstancePlan(context_wrap(AZURE_PLAN_2))
    assert azure.name is None
    assert azure.product is None
    assert azure.publisher == "Red Hat"
    assert azure.raw == '{"name": "", "product": "", "publisher": "Red Hat"}'

    azure = AzureInstancePlan(context_wrap(AZURE_PLAN_3))
    assert azure.name is None
    assert azure.product is None
    assert azure.publisher is None
    assert azure.raw == '{"name": "", "product": "", "publisher": ""}'


def test_doc_examples():
    env = {
        'azure_id': AzureInstanceID(context_wrap(AZURE_ID_1)),
        'azure_plan': AzureInstancePlan(context_wrap(AZURE_PLAN_DOC)),
        'azure_type': AzureInstanceType(context_wrap(AZURE_TYPE_DOC)),
    }
    failed, total = doctest.testmod(azure_instance, globs=env)
    assert failed == 0


def test_azure_public_ipv4():
    with pytest.raises(SkipComponent):
        AzurePublicIpv4Addresses(context_wrap(AZURE_LB_ERR1))

    with pytest.raises(SkipComponent):
        AzurePublicIpv4Addresses(context_wrap(AZURE_LB_ERR2))

    with pytest.raises(SkipComponent):
        AzurePublicIpv4Addresses(context_wrap(AZURE_LB_ERR3))

    with pytest.raises(ParseException):
        AzurePublicIpv4Addresses(context_wrap(AZURE_LB_ERR4))

    with pytest.raises(SkipComponent):
        AzurePublicIpv4Addresses(context_wrap(AZURE_LB_ERR5))

    with pytest.raises(SkipComponent):
        AzurePublicIpv4Addresses(context_wrap(AZURE_LB_ERR6))

    azure = AzurePublicIpv4Addresses(context_wrap(AZURE_LB_1))
    assert azure[0] == "137.116.118.209"



#!/usr/bin/env python
"""
The inspect module allows you to execute an insights component
(parser, combiner, rule or datasource) dropping you into an
ipython session where you can inspect the outcome.


>>> insights-inspect insights.parsers.hostname.Hostname
IPython Console Usage Info:
Enter 'Hostname.' and tab to get a list of properties
Example:
In [1]: Hostname.<property_name>
Out[1]: <property value>
To exit ipython enter 'exit' and hit enter or use 'CTL D'
Python 3.6.6 (default, Jul 19 2018, 14:25:17)
Type "copyright", "credits" or "license" for more information.
IPython 5.8.0 -- An enhanced Interactive Python.
?         -> Introduction and overview of IPython's features.
%quickref -> Quick reference.
help      -> Python's own help system.
object?   -> Details about 'object', use 'object??' for extra details.
In [1]: Hostname.fqdn
Out[1]: 'lhuett.usersys.redhat.com'"


>>> insights-inspect insights.combiners.hostname.hostname
IPython Console Usage Info:
Enter 'hostname.' and tab to get a list of properties
Example:
In [1]: hostname.<property_name>
Out[1]: <property value>
To exit ipython enter 'exit' and hit enter or use 'CTL D'
Python 3.6.6 (default, Jul 19 2018, 14:25:17)
Type "copyright", "credits" or "license" for more information.
IPython 5.8.0 -- An enhanced Interactive Python.
?         -> Introduction and overview of IPython's features.
%quickref -> Quick reference.
help      -> Python's own help system.
object?   -> Details about 'object', use 'object??' for extra details.
In [1]: hostname.fqdn
Out[1]: 'lhuett.usersys.redhat.com'


>>> insights-inspect insights.specs.Specs.hostname
IPython Console Usage Info:
Enter 'hostname.' and tab to get a list of properties
Example:
In [1]: hostname.<property_name>
Out[1]: <property value>
To exit ipython enter 'exit' and hit enter or use 'CTL D'
Python 3.6.6 (default, Jul 19 2018, 14:25:17)
Type "copyright", "credits" or "license" for more information.
IPython 5.8.0 -- An enhanced Interactive Python.
?         -> Introduction and overview of IPython's features.
%quickref -> Quick reference.
help      -> Python's own help system.
object?   -> Details about 'object', use 'object??' for extra details.
In [1]: hostname.cmd
Out[1]: '/usr/bin/hostname -f'


>>> insights-inspect examples.rules.bash_version.report
IPython Console Usage Info:
Enter 'report.' and tab to get a list of properties
Example:
In [1]: report.<property_name>
Out[1]: <property value>
To exit ipython enter 'exit' and hit enter or use 'CTL D'
Python 3.6.6 (default, Jul 19 2018, 14:25:17)
Type "copyright", "credits" or "license" for more information.
IPython 5.8.0 -- An enhanced Interactive Python.
?         -> Introduction and overview of IPython's features.
%quickref -> Quick reference.
help      -> Python's own help system.
object?   -> Details about 'object', use 'object??' for extra details.
In [1]: report.values()
Out[1]: dict_values([0:bash-4.2.46-31.el7, 'pass', 'BASH_VERSION'])
In [2]: broker.keys()
Out[2]: dict_keys([<class 'insights.core.context.HostArchiveContext'>,
<insights.core.spec_factory.glob_file object at 0x7f0303c93390>,
<insights.core.spec_factory.head object at 0x7f0303cabef0>,
insights.specs.Specs.installed_rpms,
<class 'insights.parsers.installed_rpms.InstalledRpms'>,
<function report at 0x7f0303c679d8>])
In [3]: import insights
In [4]: p = broker[insights.parsers.installed_rpms.InstalledRpms]
In [5]: a = p.get_max('bash')
In [6]: a.nevra
Out[5]: 'bash-0:4.2.46-31.el7.x86_64'
"""

from __future__ import print_function
import argparse
import logging
import os
import sys
import yaml

from contextlib import contextmanager

from insights import (apply_configs, dr, extract, HostContext,
                      load_default_plugins)
from insights.core import filters
from insights.core.hydration import initialize_broker
from IPython import embed
from IPython.terminal.embed import InteractiveShellEmbed


try:
    import colorama as C
    C.init()
except:
    class Pass(object):
        def __getattr__(self, name):
            return ""

    class C(object):
        Fore = Pass()
        Style = Pass()


def get_ipshell():
    banner = 'Starting IPython Interpreter Now \n'
    exit_msg = '\nExiting IPython Interpreter Now'

    try:
        return InteractiveShellEmbed(banner1=banner, exit_msg=exit_msg)
    except:
        print("*** Error initializing colorized shell, failing over to non-colorized shell ***\n\n")
        return embed


def parse_args():
    p = argparse.ArgumentParser("Insights component runner.")
    p.add_argument("-c", "--config", help="Configure components.")
    p.add_argument("-D", "--debug", action="store_true", help="Show debug level information.")
    p.add_argument("--nofilters", action="store_true", help="Turn datasource filters off")
    p.add_argument("component", nargs=1, help="Component to inspect.")
    p.add_argument("archive", nargs="?", help="Archive or directory to analyze.")
    return p.parse_args()


def configure_logging(debug):
    if debug:
        logging.basicConfig(level=logging.DEBUG, stream=sys.stderr)


def configure(config):
    if config:
        with open(config) as f:
            apply_configs(yaml.safe_load(f))


def get_component(fqdn):
    return dr.get_component(fqdn)


@contextmanager
def create_broker(root=None):
    if not root:
        broker = dr.Broker()
        broker[HostContext] = HostContext()
        yield broker
    else:
        def from_dir(d):
            # ctx is returned here, but its already in the broker so not needed
            _, broker = initialize_broker(d)
            return broker

        if os.path.isdir(root):
            yield from_dir(root)
        else:
            with extract(root) as ex:
                yield from_dir(ex.tmp_dir)


def dump_error(component, broker):
    if component in broker.exceptions:
        for ex in broker.exceptions[component]:
            print(broker.tracebacks[ex], file=sys.stderr)

    if component in broker.missing_requirements:
        missing = broker.missing_requirements[component]
        required = missing[0]
        at_least_one = missing[1]

        buf = sys.stderr

        print("Missing Dependencies:", file=buf)
        if required:
            print("    Requires:", file=buf)
            for d in required:
                print("        %s" % dr.get_name(d), file=buf)
        if at_least_one:
            for alo in at_least_one:
                print("    At Least One Of:", file=buf)
                for d in alo:
                    print("        %s" % dr.get_name(d), file=buf)


def run(component, archive=None):
    with create_broker(archive) as broker:
        value = dr.run(component, broker=broker).get(component)
        name = dr.get_name(component).rsplit(".", 1)[1]
        vars()[name] = value
        if value:
            print(C.Fore.BLUE + "\nIPython Console Usage Info:\n" + C.Style.RESET_ALL, file=sys.stderr)
            print("Enter '{}.' and tab to get a list of properties ".format(name))
            print("Example:")
            print("In [1]: {}.<property_name>".format(name))
            print("Out[1]: <property value>\n")
            print("To exit ipython enter 'exit' and hit enter or use 'CTL D'\n")
            ipshell = get_ipshell()
            ipshell()
        else:
            dump_error(component, broker)
            return sys.exit(1)


def main():
    args = parse_args()
    configure_logging(args.debug)
    configure(args.config)
    load_default_plugins()
    component = get_component(args.component[0])
    if not component:
        print("Component not found: %s" % args.component[0], file=sys.stderr)
        sys.exit(1)
    if args.nofilters:
        filters.ENABLED = False

    run(component, archive=args.archive)


if __name__ == "__main__":
    main()



"""
Parsers for Microsoft Defender for Endpoint configuration
=========================================================
MdatpManaged - file ``/etc/opt/microsoft/mdatp/managed/mdatp_managed.json``
---------------------------------------------------------------------------
"""

from insights import parser
from insights.specs import Specs
from insights.core import JSONParser


@parser(Specs.mdatp_managed)
class MdatpManaged(JSONParser):
    """
    Class for parsing the file: ``/etc/opt/microsoft/mdatp/managed/mdatp_managed.json``.

    This configuration file provides security settings for Microsoft Defender for Endpoint.

    .. note::
        Please refer to the super-class :class:`insights.core.JSONParser`
        for additional information on attributes and methods.

    Sample input data::

        {
           "exclusionSettings":{
             "exclusions":[
                {
                   "$type":"excludedPath",
                   "isDirectory":false,
                   "path":"/var/log/system.log<EXAMPLE DO NOT USE><EXCLUDED IN ALL SCENARIOS>",
                   "scopes": [
                      "epp", "global"
                   ]
                },
                {
                   "$type":"excludedFileName",
                   "name":"/bin/cat<EXAMPLE DO NOT USE><NO SCOPE PROVIDED - GLOBAL CONSIDERED>"
                }
             ],
             "mergePolicy":"admin_only"
           }
        }

    Examples:
        >>> 'exclusionSettings' in mdatp
        True
        >>> 'mergePolicy' in mdatp['exclusionSettings']
        True
        >>> mdatp['exclusionSettings']['mergePolicy'] == "admin_only"
        True
    """
    pass



"""
LocaleCtlStatus - command ``localectl status``
==============================================
Parser the output of localectl status command.

"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.localectl_status)
class LocaleCtlStatus(CommandParser, dict):
    """
    Reads the output of `localectl status` command

    Example output::

           System Locale: LANG=en_US.UTF-8
               VC Keymap: us
              X11 Layout: us

    Examples::
        >>> type(localectl_status)
        <class 'insights.parsers.localectl.LocaleCtlStatus'>
        >>> localectl_status['System Locale'] == 'LANG=en_US.UTF-8'
        True
        >>> localectl_status['VC Keymap'] == 'us'
        True
        >>> localectl_status['X11 Layout'] == 'us'
        True
    """
    def parse_content(self, content):
        for line in content:
            if ': ' in line:
                key, val = [_l.strip() for _l in line.split(': ', 1)]
                self[key] = val

        if not self:
            raise SkipComponent



"""
Custom datasources to get the device names.
"""

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.parsers.blkid import BlockIDInfo
from insights.combiners.virt_what import VirtWhat


@datasource(BlockIDInfo, VirtWhat, HostContext)
def physical_devices(broker):
    """
    This datasource get the name of the physical_devices.

    Returns:
        list: the names of the physical_devices

    Raises:
        SkipComponent: there is physical_device
    """
    blockdinfo = broker[BlockIDInfo]
    virtwhat = broker[VirtWhat]
    result = []
    if virtwhat.is_physical:
        for item in blockdinfo.data:
            device_name = item["NAME"]
            if device_name.startswith("/dev/sd") or device_name.startswith("/dev/hd"):
                main_dev_name = ''.join([i for i in device_name if not i.isdigit()])
                result.append(main_dev_name)
    if result:
        return sorted(list(set(result)))
    raise SkipComponent("No physical device")



# coding:UTF-8
from insights.parsr.iniparser import parse_doc
from insights.parsr.query import last
from six import PY2
from copy import deepcopy


DATA = """
[DEFAULT]
trooper_guns = miss

[global]
logging=debug
log=/var/logs/sample.log

# Keep this info secret
[secret_stuff]
username=dvader
password=luke_is_my_son

[facts]
major_vulnerability=ray-shielded particle exhaust vent
vader = definitely Luke's
    father

[settings]
music=The Imperial March
color=blue
#blah
color=black
accuracy=0
banks=0 1 2

[novalue]
the_force

""".strip()

DATA_UNICODE_TEST = """
[global]
secret-name = "vsphere-creds"
secret-namespace = kube-system
insecure-flag = 1

[workspace]
datacenter = 1-测试部
folder = "/1-测试部/xxxxxxxxx"
""".strip()


def test_iniparser():
    res = parse_doc(DATA, None)
    assert len(res) == 5


def test_hanging_indent():
    res = parse_doc(DATA, None)
    assert res["facts"]["vader"][0].value == "definitely Luke's father"


def test_defaults():
    res = parse_doc(DATA, None)
    assert res["facts"]["trooper_guns"][0].value == "miss"


def test_multiple_values():
    res = parse_doc(DATA, None)
    assert len(res["settings"]["color"]) == 2
    assert res["settings"]["accuracy"][0].value == "0"
    assert res["settings"]["banks"][0].value == "0 1 2"


def test_no_value():
    res = parse_doc(DATA, None)
    assert res["novalue"]["the_force"][0].value is None


def test_unicode():
    res = parse_doc(DATA_UNICODE_TEST, None)
    assert res["global"]["insecure-flag"][last].value == "1"
    assert res["global"]["secret-name"][last].value == '"vsphere-creds"'
    assert res["global"]["secret-namespace"][last].value == "kube-system"

    if PY2:
        assert res["workspace"]["datacenter"][last].value == "1-?????????"
        assert res["workspace"]["folder"][last].value == '"/1-?????????/xxxxxxxxx"'
    else:
        assert res["workspace"]["datacenter"][last].value == "1-???"
        assert res["workspace"]["folder"][last].value == '"/1-???/xxxxxxxxx"'


def test_deepcopy():
    res = parse_doc(DATA, None)

    # This should not throw an exception
    deepcopy(res)



"""
Filtering
=========
"""

import logging

logger = logging.getLogger(__name__)


class AllowFilter(object):
    """
    Class for filtering per allow list.
    """

    def parse_line(self, line, **kwargs):
        # filter line as per the allow list specified by plugins
        if not line:
            return line
        allowlist = kwargs.get('allowlist', {})
        if allowlist:
            for a_key in list(allowlist.keys()):  # copy keys to avoid RuntimeError
                # keep line when any filter match
                # FIXME:
                # Considering performance, didn't handle multiple filters in one same line
                if a_key in line:
                    allowlist[a_key] -= 1
                    # stop checking it when enough lines contain the key were found
                    allowlist.pop(a_key) if allowlist[a_key] == 0 else None
                    return line
        # discard line when none filters found

    def generate_report(self, report_dir, archive_name):
        pass  # pragma: no cover

    @staticmethod
    def filter_content(lines, allowlist):
        """
        Filter content based on allowlist.

        When a key of allowlist is found in a line, it is added to the result
        list. The key is removed from the allowlist when enough lines
        containing the key were found.
        When the allowlist is empty, an empty result is returned.

        The lines are processed in reverse order.   But the processed result
        is returned in the original order.

        :param lines: list of lines
        :param allowlist: dictionary of allowlist
        :return: list of lines
        """
        allowlist = dict(allowlist)  # copy it to avoid write back
        result = []
        for idx in range(len(lines) - 1, -1, -1):
            for a_key in list(allowlist.keys()):  # copy keys to avoid RuntimeError
                if a_key in lines[idx]:
                    allowlist[a_key] -= 1
                    # stop checking it when enough lines contain the key were found
                    allowlist.pop(a_key) if allowlist[a_key] == 0 else None
                    result.append(lines[idx])
                    # stop checking other keys when one key is found
                    # it's sometimes not fair to other keys, but it's
                    # the best we can do for performance
                    break
        # Return the result in right order
        result.reverse()
        return result



"""
LsRsyslogErrorfile - command ``ls -ln <rsyslog errorfile path>``
================================================================

Parser for the output of ``ls -ln <rsyslog errorfile path>`` command
"""

from insights.core import Parser, ls_parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_rsyslog_errorfile)
class LsRsyslogErrorfile(Parser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.ls.LSlHFiles` instead.

    The ``ls -ln <rsyslog errorfile path>`` command provides information for the
    rsyslog errorfile paths.

    Attributes:
        entries (dict): Dictionary of keys with values in dict.

    Sample directory list collected::

        -rw-r--r--. 1 0 0   9 Mar 15 17:16 /var/log/omelasticsearch.log
        -rw-r--r--. 1 0 0 176 Mar 22 15:10 /var/log/rsyslog/es-errors1.log

    Examples:
        >>> type(rsyslog_errorfile)
        <class 'insights.parsers.ls_rsyslog_errorfile.LsRsyslogErrorfile'>
        >>> len(rsyslog_errorfile.entries)
        2
        >>> rsyslog_errorfile.entries.get('/var/log/omelasticsearch.log').get('size')
        9
    """

    def __init__(self, *args, **kwargs):
        deprecated(LsRsyslogErrorfile, "Please use insights.parsers.ls.LSlHFiles instead.", "3.7.0")

    def parse_content(self, content):
        parsed_content = []
        for line in content:
            if 'No such file or directory' not in line:
                parsed_content.append(line)
        if not parsed_content:
            raise ParseException('Error: {0}', content)
        ls_data = ls_parser.parse(parsed_content, '').get('')
        self.entries = ls_data.get('entries')



"""
NginxErrorLog - file ``/var/log/nginx/error.log``
=================================================

Module for parsing log file of nginx web server.

.. note::
    Please refer to the super-class :py:class:`insights.core.LogFileOutput`
    for more usage information.
"""

from insights.core import ContainerParser, LogFileOutput
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.nginx_error_log)
class NginxErrorLog(LogFileOutput):
    """
    Class for parsing ``/var/log/nginx/error.log`` file.

    Sample log contents::

        2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"
        2022/04/02 04:08:46 [warn] 1592#1592: *697439 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/3/25/0000003253 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=50 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"
        2022/04/02 04:09:27 [warn] 1593#1593: *697455 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/4/25/0000003254 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=53 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"
        2022/04/02 05:30:59 [warn] 1591#1591: *698917 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/5/25/0000003255 while reading upstream, client: ::1, server: _, request: "GET /api/v2/inventories/61/script/?hostvars=1&towervars=1&all=1 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "localhost:44443"
        2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com", referrer: "https://host.example.com/"
        2022/04/03 00:02:28 [warn] 1591#1591: *719136 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/7/25/0000003257 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/organizations/?page_size=999 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"

    The ``get`` method breaks up log lines into the following field:

    * **raw_message(str)** - the original unparsed line

    Each line of the resultant list is a dictionary with this field.

    Examples:

        >>> len(nginx_error_log.lines)
        6
        >>> nginx_error_log.lines[0]
        '2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"'
        >>> '711881' in nginx_error_log
        True
        >>> nginx_error_log.get('711881')
        [{'raw_message': '2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com", referrer: "https://host.example.com/"'}]
        >>> from datetime import datetime
        >>> list(nginx_error_log.get_after(datetime(2022, 4, 2, 17, 0, 0)))[0]['raw_message']
        '2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com", referrer: "https://host.example.com/"'
    """
    time_format = '%Y/%m/%d %H:%M:%S'


@parser(Specs.container_nginx_error_log)
class ContainerNginxErrorLog(ContainerParser, NginxErrorLog):
    """
    Class for parsing ``/var/log/nginx/error.log`` file from containers.

    Sample log contents::

        2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"
        2022/04/02 04:08:46 [warn] 1592#1592: *697439 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/3/25/0000003253 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=50 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"
        2022/04/02 04:09:27 [warn] 1593#1593: *697455 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/4/25/0000003254 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=53 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"
        2022/04/02 05:30:59 [warn] 1591#1591: *698917 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/5/25/0000003255 while reading upstream, client: ::1, server: _, request: "GET /api/v2/inventories/61/script/?hostvars=1&towervars=1&all=1 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "localhost:44443"
        2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com", referrer: "https://host.example.com/"
        2022/04/03 00:02:28 [warn] 1591#1591: *719136 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/7/25/0000003257 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/organizations/?page_size=999 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"

    The ``get`` method breaks up log lines into the following field:

    * **raw_message(str)** - the original unparsed line

    Each line of the resultant list is a dictionary with this field.

    Examples:

        >>> len(container_nginx_error_log.lines)
        6
        >>> container_nginx_error_log.container_id
        '2869b4e2541c'
        >>> container_nginx_error_log.image
        'reg.host.example.com/ubi8/nginx-120'
        >>> container_nginx_error_log.lines[0]
        '2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com"'
        >>> '711881' in container_nginx_error_log
        True
        >>> container_nginx_error_log.get('711881')
        [{'raw_message': '2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com", referrer: "https://host.example.com/"'}]
        >>> from datetime import datetime
        >>> list(container_nginx_error_log.get_after(datetime(2022, 4, 2, 17, 0, 0)))[0]['raw_message']
        '2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "host.example.com", referrer: "https://host.example.com/"'
    """
    pass



"""
RHN Logs -  Files ``/var/log/rhn/*.log``
========================================

Modules for parsing the content of log files under ``/rhn-logs/rhn`` directory
in spacewalk-debug or sosreport archives of Satellite 5.x.

.. note::
    Please refer to the super-class :class:`insights.core.LogFileOutput`
"""
from datetime import datetime
from .. import parser, LogFileOutput
import re
from insights.specs import Specs


@parser(Specs.rhn_taskomatic_daemon_log)
class TaskomaticDaemonLog(LogFileOutput):
    """Class for parsing the ``rhn_taskomatic_daemon.log`` file.

    .. note::
        Because of the need to get the datetime of the last log, please DO NOT
        filter it.

    Attributes:
        lines (list): All lines captured in this file.
        last_log_date (datetime): The last log datetime get from the last line.

    Examples:
        >>> td_log = shared[TaskomaticDaemonLog]
        >>> td_log.file_path
        'var/log/rhn/rhn_taskomatic_daemon.log'
        >>> td_log.get('two')[0]['raw_message']
        'Log file line two'
        >>> 'three' in td_log
        True
        >>> td_log.last_log_date
        2016-05-18 15:13:40
    """
    time_format = '%Y/%m/%d %H:%M:%S'

    def parse_content(self, content):
        """
        Once the logs are parsed, retrieve the last log date from the last
        line which has a complete timestamp in the third field.
        """
        super(TaskomaticDaemonLog, self).parse_content(content)
        self.last_log_date = None

        for l in reversed(self.lines):
            l_sp = l.split('|')
            if len(l_sp) >= 3:
                try:
                    self.last_log_date = datetime.strptime(
                            l_sp[2].strip(), self.time_format)
                    break
                except Exception:
                    continue


@parser(Specs.rhn_server_xmlrpc_log)
class ServerXMLRPCLog(LogFileOutput):
    """Class for parsing the ``rhn_server_xmlrpc.log`` file.

    Sample log line::

        2016/04/11 05:52:01 -04:00 23630 10.4.4.17: xmlrpc/registration.welcome_message('lang: None',)

    Attributes:
        lines (list): All lines captured in this file.
        last (dict): Dict of the last log line.

    Examples:
        >>> log = shared[ServerXMLRPCLog]
        >>> log.file_path
        'var/log/rhn/rhn_server_xmlrpc.log'
        >>> log.get('two')
        [{'timestamp':'2016/04/11 05:52:01 -04:00',
          'datetime': datetime(2016, 04, 11, 05, 52, 01),
          'pid': '23630',
          'client_ip': '10.4.4.17',
          'module': 'xmlrpc',
          'function': 'registration.welcome_message',
          'client_id': None,
          'args': "'lang: None'",
          'raw_message': "...two..."}]
        >>> log.last
        [{'timestamp':'2016/04/11 05:52:01 -04:00',
          'datetime': datetime(2016, 04, 11, 05, 52, 01),
          'pid': '23630',
          'client_ip': '10.4.4.17',
          'module': 'xmlrpc',
          'function': 'registration.welcome_message',
          'client_id': None,
          'args': "'lang: None'",
          'raw_message': "..."}]
    """
    time_format = '%Y/%m/%d %H:%M:%S'

    _LINE_STR = r"^(?P<timestamp>\d{4}/\d{2}/\d{2} \d{2}:\d{2}:\d{2} " + \
        r"[+-]?\d{2}:\d{2}) (?P<pid>\d+) (?P<client_ip>\S+): " + \
        r"(?P<module>\w+)/(?P<function>[\w.-]+)" + \
        r"(?:\((?:(?P<client_id>\d+), ?)?(?P<args>.*?),?\))?$"
    _LINE_RE = re.compile(_LINE_STR)

    def parse_content(self, content):
        """
        Parse the logs as its super class LogFileOutput.
        And get the last complete log. If the last line is not complete,
        then get from its previous line.
        """
        super(ServerXMLRPCLog, self).parse_content(content)
        self.last = None

        msg_info = {}
        for l in reversed(self.lines):
            msg_info = self._parse_line(l)
            # assume parse is successful if we got an IP address
            if 'client_ip' in msg_info:
                break
        # Get the last one even if it didn't parse.
        self.last = msg_info

    def _parse_line(self, line):
        """
        Parse a log line using the XMLRPC regular expression into a dict.
        All data will be in fields, and the raw log line is stored in
        'raw_message'.
        """
        msg_info = dict()
        msg_info['raw_message'] = line

        match = self._LINE_RE.search(line)
        if match:
            msg_info.update(match.groupdict())
            # Try converting the time stamp but move on if it fails
            try:
                stamp = match.group('timestamp')
                # Cannot guess time zone from e.g. '+01:00', so strip timezone
                msg_info['datetime'] = datetime.strptime(
                    stamp[0:19], self.time_format)
            except ValueError:
                pass

        return msg_info


@parser(Specs.rhn_search_daemon_log)
class SearchDaemonLog(LogFileOutput):
    """
    Class for parsing the ``/var/log/rhn/search/rhn_search_daemon.log`` file.

    Sample log contents::

        STATUS | wrapper  | 2013/01/28 14:41:58 | --> Wrapper Started as Daemon
        STATUS | wrapper  | 2013/01/28 14:41:58 | Launching a JVM...
        INFO   | jvm 1    | 2013/01/28 14:41:59 | Wrapper (Version 3.2.1) http://wrapper.tanukisoftware.org
        STATUS | wrapper  | 2013/01/29 17:04:25 | TERM trapped.  Shutting down.

    Attributes:
        lines (list): All lines captured in this file.

    Examples:
        >>> log = shared[SearchDaemonLog]
        >>> log.file_path
        'var/log/rhn/search/rhn_search_daemon.log'
        >>> log.get('Launching a JVM')[0]['raw_message']
        'STATUS | wrapper  | 2013/01/28 14:41:58 | Launching a JVM...'
        >>> list(log.get_after(datetime(2013, 1, 29, 0, 0, 0)))[0]['raw_message']
        'STATUS | wrapper  | 2013/01/29 17:04:25 | TERM trapped.  Shutting down.'
    """
    time_format = '%Y/%m/%d %H:%M:%S'


@parser(Specs.rhn_server_satellite_log)
class SatelliteServerLog(LogFileOutput):
    """
    Class for parsing the ``var/log/rhn/rhn_server_satellite.log`` file

    Sample log contents::

        2016/11/19 01:13:35 -04:00 Downloading errata data complete
        2016/11/19 01:13:35 -04:00 Downloading kickstartable trees metadata
        2016/11/19 01:13:35 -04:00    Retrieving / parsing kickstart tree files: rhel-x86_64-server-optional-6-debuginfo (NONE RELEVANT)
        2016/11/19 01:13:39 -04:00    debug/output level: 1
        2016/11/19 01:13:39 -04:00    db:  rhnsat/<password>@rhnsat
        2016/11/19 01:13:39 -04:00
        2016/11/19 01:13:39 -04:00 Retrieving / parsing channel-families data
        2016/11/19 01:13:44 -04:00 channel-families data complete
        2016/11/19 01:13:44 -04:00
        2016/11/19 01:13:44 -04:00 RHN Entitlement Certificate sync

    Examples:
        >>> log = shared[SatelliteServerLog]
        >>> log.get('Downloading')[0['raw_message']
        '2016/11/19 01:13:35 -04:00 Downloading errata data complete', '2016/11/19 01:13:35 -04:00 Downloading kickstartable trees metadata'
        >>> list(log.set_after(datetime(2016, 11, 19, 1, 13, 44)))[0]['raw_message']
        '2016/11/19 01:13:44 -04:00 channel-families data complete', '2016/11/19 01:13:44 -04:00 ', '2016/11/19 01:13:44 -04:00 RHN Entitlement Certificate sync'
    """
    time_format = '%Y/%m/%d %H:%M:%S'



"""
Cloud Provider
==============

Combiner for Cloud information. It uses the results of the multiple parsers:

* :py:class:`insights.parsers.installed_rpms.InstalledRpms`
* :py:class:`insights.parsers.yum.YumRepoList`
* :py:class:`insights.parsers.dmidecode.DMIDecode`
* :py:class:`insights.parsers.rhsm_conf.RHSMConf`

The combiner uses these parsers determine the Cloud Provider based on a set of
criteria that is unique to each cloud provider.

Examples:
    >>> cp_aws.cloud_provider
    'aws'
    >>> cp_aws.cp_bios_version['aws'] == '4.2.amazon'
    True
    >>> cp_aws.cp_rpms['aws'] == ['rh-amazon-rhui-client-2.2.124-1.el7']
    True
    >>> cp_aws.cp_uuid['aws']
    'EC2F58AF-2DAD-C57E-88C0-A81CB6084290'
    >>> cp_aws.long_name
    'Amazon Web Services'
    >>> cp_azure.cloud_provider
    'azure'
    >>> cp_azure.cp_yum['azure'] == ['rhui-microsoft-azure-rhel7-2.2-74']
    True
    >>> cp_azure.cp_asset_tag['azure']
    '7783-7084-3265-9085-8269-3286-77'
    >>> cp_alibaba.cloud_provider
    'alibaba'
    >>> cp_alibaba.cp_manufacturer['alibaba'] == 'Alibaba Cloud'
    True
    >>> cp_ibm.cp_rhsm_server_hostname['ibm'] == 'host.networklayer.com'
    True

"""

from insights.core.filters import add_filter
from insights.core.plugins import combiner
from insights.parsers.cloud_init import CloudInitQuery
from insights.parsers.dmidecode import DMIDecode
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.rhsm_conf import RHSMConf
from insights.parsers.subscription_manager import SubscriptionManagerFacts
from insights.parsers.yum import YumRepoList

add_filter(RHSMConf, ['server', 'hostname'])
add_filter(SubscriptionManagerFacts, 'instance_id')


class CloudProviderInstance(object):
    """
    Class to represent a base cloud provider instance

    Use this base class to derive new cloud provider classes.  In each new cloud
    provider class set the particular values that will be used to detect that
    particular cloud provider.

    Attributes:
        rpm (str): RPM string in lowercase to use when searching for this cloud provider.
        yum (str): Yum repo name string in lowercase to use when searching for this cloud provider.
        bios_vendor_version (str): BIOS vendor version string in lowercase to use when searching
            for this cloud provider.
        manuf (str): Manufacturer string in lowercase to use when searching for this cloud provider.
        asset_tag (str): Asset tag string in lowercase to use when searching for this
            cloud provider.
        uuid (str): UUID string in lowercase to use when searchinf for this cloud provider.
        rhsm_hostname (str): Hostname string in lowercase to use when searching for this
            cloud provider in ``rhsm.conf``.
        cp_subman_facts (str): Cloud name found in searching SubscriptionManagerFacts for this cloud provider.
        cp_cloud_query (str): Cloud name found in searching CloudInitQuery for this cloud provider.
        cp_bios_vendor (str): BIOS vendor string value found in search for this cloud provider.
        cp_bios_version (str): BIOS version string value found in search for this cloud provider.
        cp_rpms (list): List of RPM string values found in search for this cloud provider.
        cp_yum (list): List of Yum repo name string values found in search for this cloud provider.
        cp_asset_tag (str): Asset tag string value found in search for this cloud provider.
        cp_uuid (str): UUID string value found in search for this cloud provider.
        cp_manufacturer (str): Manufacturer string value found in search for this cloud provider.
        cp_rhsm_server_hostname (str): RHSM server hostname string value found in search for
            this cloud provider.
    """

    def __init__(
        self,
        rpms=None,
        dmidcd=None,
        yum_repos=None,
        rhsm_cfg=None,
        cloud_query=None,
        subman_facts=None,
    ):
        self._rpms = rpms
        self._dmidcd = dmidcd
        self._yum_repos = yum_repos
        self._rhsm_cfg = rhsm_cfg
        self._cloud_query = cloud_query
        self._subman_facts = subman_facts
        self.rpm = ''
        self.yum = ''
        self.bios_vendor_version = ''
        self.manuf = ''
        self.asset_tag = ''
        self.uuid = ''
        self.rhsm_hostname = ''
        self.cp_bios_vendor = ''
        self.cp_bios_version = ''
        self.cp_rpms = []
        self.cp_yum = []
        self.cp_asset_tag = ''
        self.cp_uuid = ''
        self.cp_manufacturer = ''
        self.cp_rhsm_server_hostname = ''
        self.cp_cloud_query = ''
        self.cp_subman_facts = ''

    def _get_cp_bios_vendor(self, vendor_version):
        """str: Returns BIOS vendor string if it matches ``vendor_version``"""
        vendor = ''
        if self._dmidcd and self._dmidcd.bios:
            vendor = (
                self._dmidcd.bios.get('vendor')
                if vendor_version and vendor_version in self._dmidcd.bios.get('vendor', '').lower()
                else ''
            )
        return vendor

    def _get_cp_bios_version(self, vendor_version):
        """str: Returns BIOS version string if it matches ``vendor_version``"""
        version = ''
        if self._dmidcd and self._dmidcd.bios:
            version = (
                self._dmidcd.bios.get('version')
                if vendor_version and vendor_version in self._dmidcd.bios.get('version', '').lower()
                else ''
            )
        return version

    def _get_rpm_cp_info(self, rpm):
        """list: Returns list of RPMs matching ``rpm``"""
        found_rpms = []
        if rpm and self._rpms:
            for key, val in self._rpms.packages.items():
                for v in val:
                    if rpm in v.package.lower():
                        found_rpms.append(v.package)
        return found_rpms

    def _get_cp_from_manuf(self, manuf):
        """str: Returns manufacturer string if it matches ``manuf``"""
        manufacturer = ''
        if self._dmidcd and self._dmidcd.system_info:
            manufacturer = (
                self._dmidcd.system_info.get('manufacturer')
                if manuf == self._dmidcd.system_info.get('manufacturer', '').lower()
                else ''
            )
        return manufacturer

    def _get_cp_from_yum(self, repo_name):
        """list: Returns list of Yum repos matching ``repo_name``"""
        found_repos = []
        if self._yum_repos and hasattr(self._yum_repos, 'data'):
            found_repos = [
                repo.get('id').lower()
                for repo in self._yum_repos.data
                if repo_name and repo_name in repo.get('id', '').lower()
            ]
        return found_repos

    def _get_cp_from_rhsm_conf(self, rhsm_server_hostname):
        """str: Returns rhsm server hostname string if it matches ``rhsm_server_hostname``"""
        server_hostname = ''
        if self._rhsm_cfg and 'server' in self._rhsm_cfg and 'hostname' in self._rhsm_cfg['server']:
            hostname = self._rhsm_cfg.get('server', 'hostname')
            if hostname and hostname.lower().strip().endswith(rhsm_server_hostname):
                server_hostname = hostname
        return server_hostname

    def _get_cp_from_asset_tag(self, asset_tag):
        """str: Returns asset tag string if it matches ``asset_tag``"""
        tag = ''
        if self._dmidcd and hasattr(self._dmidcd, 'data'):
            ch_info = self._dmidcd.data.get('chassis_information', [])
            if ch_info:
                tag = (
                    ch_info[0].get('asset_tag')
                    if asset_tag and asset_tag == ch_info[0].get('asset_tag', '')
                    else ''
                )
        return tag

    def _get_cp_from_uuid(self, uuid):
        """str: Returns UUID string if it matches ``uuid``"""
        found_uuid = ''
        if self._dmidcd and self._dmidcd.system_info:
            found_uuid = (
                self._dmidcd.system_info.get('uuid')
                if uuid
                and self._dmidcd.system_info.get('uuid', '').lower().strip().startswith(uuid)
                else ''
            )
        return found_uuid

    def _get_cp_from_cloud_init_query(self, name):
        """
        str: Returns cloud_query name if CloudInitQuery is available and
        ``name`` match its cloud_name.
        """
        if self._cloud_query:
            if name in self._cloud_query.cloud_name:
                return self._cloud_query.cloud_name
        return ''

    def _get_cp_from_subman_facts(self, name):
        """
        str: Returns cloud_query name if SubscriptionManagerFacts is available and
        ``name`` match its '_instance_id' key.
        """
        if self._subman_facts:
            if "{0}_instance_id".format(name) in self._subman_facts:
                return name
        return ''

    @property
    def name(self):
        """str: Short cloud provider class name or ID"""
        return self._NAME

    @property
    def long_name(self):
        """str: Long cloud provider name"""
        return self._LONG_NAME


class GoogleCloudProvider(CloudProviderInstance):
    """
    Class to identify Google Cloud provider

    Google CP can be identified by RPM and BIOS vendor/version
    """

    _NAME = 'gcp'
    _LONG_NAME = 'Google Cloud Platform'

    def __init__(self, *args, **kwargs):
        super(GoogleCloudProvider, self).__init__(*args, **kwargs)
        self.bios_vendor_version = 'google'
        self.rpm = 'google-rhui-client'
        self.cp_bios_vendor = self._get_cp_bios_vendor(self.bios_vendor_version)
        self.cp_bios_version = self._get_cp_bios_version(self.bios_vendor_version)
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)
        # https://github.com/canonical/cloud-init/blob/main/tests/integration_tests/modules/test_combined.py#L528
        self.cp_cloud_query = self._get_cp_from_cloud_init_query('gce')
        self.cp_subman_facts = self._get_cp_from_subman_facts(self.name)


class AlibabaCloudProvider(CloudProviderInstance):
    """
    Class to identify Alibaba Cloud provider

    Alibaba CP can be identified by manufacturer
    """

    _NAME = 'alibaba'
    _LONG_NAME = 'Alibaba Cloud'

    def __init__(self, *args, **kwargs):
        super(AlibabaCloudProvider, self).__init__(*args, **kwargs)
        self.manuf = 'alibaba cloud'
        self.cp_manufacturer = self._get_cp_from_manuf(self.manuf)
        # https://github.com/canonical/cloud-init/blob/main/tests/unittests/sources/test_aliyun.py#L253
        self.cp_cloud_query = self._get_cp_from_cloud_init_query('aliyun')


class AmazonCloudProvider(CloudProviderInstance):
    """
    Class to identify Amazon Cloud provider

    Amazon CP can be identified by RPM, BIOS verndor/version,
    and system UUID
    """

    _NAME = 'aws'
    _LONG_NAME = 'Amazon Web Services'

    def __init__(self, *args, **kwargs):
        super(AmazonCloudProvider, self).__init__(*args, **kwargs)
        self.bios_vendor_version = 'amazon'
        self.rpm = 'rh-amazon-rhui-client'
        self.uuid = 'ec2'
        self.asset_tag = 'Amazon EC2'
        self.cp_bios_vendor = self._get_cp_bios_vendor(self.bios_vendor_version)
        self.cp_bios_version = self._get_cp_bios_version(self.bios_vendor_version)
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)
        self.cp_uuid = self._get_cp_from_uuid(self.uuid)
        self.cp_asset_tag = self._get_cp_from_asset_tag(self.asset_tag)
        self.cp_cloud_query = self._get_cp_from_cloud_init_query(self._NAME)
        self.cp_subman_facts = self._get_cp_from_subman_facts(self._NAME)


class AzureCloudProvider(CloudProviderInstance):
    """
    Class to identify Azure Cloud provider

    Azure CP can be identified by RPM, Yum repo, and system asset tag
    """

    _NAME = 'azure'
    _LONG_NAME = 'Microsoft Azure'

    def __init__(self, *args, **kwargs):
        super(AzureCloudProvider, self).__init__(*args, **kwargs)
        self.yum = 'rhui-microsoft-azure'
        self.asset_tag = '7783-7084-3265-9085-8269-3286-77'
        self.cp_yum = self._get_cp_from_yum(self.yum)
        self.cp_asset_tag = self._get_cp_from_asset_tag(self.asset_tag)
        self.cp_cloud_query = self._get_cp_from_cloud_init_query(self._NAME)
        self.cp_subman_facts = self._get_cp_from_subman_facts(self._NAME)
        # Do NOT check RPM for Azure anymore
        # self.rpm = 'walinuxagent'
        # self.cp_rpms = self._get_rpm_cp_info(self.rpm)


class IBMCloudProvider(CloudProviderInstance):
    """
    Class to identify IBM Cloud provider

    IBM CP can be identified by rhsm.conf server hostname setting
    """

    _NAME = 'ibm'
    _LONG_NAME = 'IBM Cloud'

    def __init__(self, *args, **kwargs):
        super(IBMCloudProvider, self).__init__(*args, **kwargs)
        self.rhsm_server_hostname = 'networklayer.com'
        self.cp_rhsm_server_hostname = self._get_cp_from_rhsm_conf(self.rhsm_server_hostname)
        # https://github.com/canonical/cloud-init/blob/main/tests/unittests/sources/test_ibmcloud.py#L426
        self.cp_cloud_query = self._get_cp_from_cloud_init_query('ibmcloud')


class OracleCloudProvider(CloudProviderInstance):
    """
    Class to identify Oracle Cloud provider

    Oracle CP can be identified by RPM for now.
    """

    _NAME = 'oci'
    _LONG_NAME = 'Oracle Cloud Infrastructure'

    def __init__(self, *args, **kwargs):
        super(OracleCloudProvider, self).__init__(*args, **kwargs)
        self.rpm = 'oracle-cloud-agent'
        self.cp_rpms = self._get_rpm_cp_info(self.rpm)
        # https://github.com/canonical/cloud-init/blob/main/tests/unittests/sources/test_oracle.py#L332
        self.cp_cloud_query = self._get_cp_from_cloud_init_query('oracle')


@combiner(
    [InstalledRpms, DMIDecode, YumRepoList, RHSMConf, CloudInitQuery, SubscriptionManagerFacts]
)
class CloudProvider(object):
    """
    Combiner class to provide cloud vendor facts

    Attributes:
        cloud_provider (str): String representing the cloud provider that was detected.
            If none are detected then it will have the default value `None`.
        long_name (str): String representing the full name of the cloud provider that
            was detected.  If none are detected then it will have the default value `None`.
        cp_subman_facts(dict): Dictionary containing a value, for each provider,
            of cloud name gets per the key '_instance_id' of
            `SubscriptionManagerFacts`.  Value will be empty if no matches are found.
        cp_cloud_query(dict): Dictionary containing a value, for each provider,
            of `CloudInitQuery.cloud_name`.  Value will be empty if no matches are found.
        cp_bios_vendor (dict): Dictionary containing a value , for each provider,
            of Bios vendor used to determine cloud provider. Each providers value will be
            empty if none found
        cp_bios_version (dict): Dictionary containing a value, for each provider,
            of Bios version used to determine cloud provider. Each providers value will be
            empty if none found
        cp_rpms (dict): Dictionary containing a list, for each provider, of rpm information
            used to determine cloud provider. Each providers list will be empty if no matches
            found
        cp_yum (dict): Dictionary containing a list, for each provider, of yum repo information
            used to determine cloud provider. Each providers list will be empty if no matches
            found
        cp_asset_tag (dict): Dictionary containing a value, for each provider, of rpm information
            used to determine cloud provider. Each providers value will be empty if no matches
            found
        cp_uuid (dict): Dictionary containing a value, for each provider, of uuid information
            used to determine cloud provider. Each providers value will be empty if no matches
            are found
        cp_manufacturer (dict): Dictionary containing a value, for each provider, of system
            information used to determine cloud provider. Provider value will be empty if no
            matches are found.
        cp_rhsm_server_hostname (dict): Dictionary containing a value, for each provider,
            of rhsm.conf server hostnames.  Value will be empty if no matches are found.
    """

    ALIBABA = AlibabaCloudProvider._NAME
    """Alibaba Cloud Provider short name"""

    AWS = AmazonCloudProvider._NAME
    """AWS Cloud Provider short name"""

    AZURE = AzureCloudProvider._NAME
    """AZURE Cloud Provider short name"""

    GOOGLE = GoogleCloudProvider._NAME
    """GOOGLE Cloud Provider short name"""

    IBM = IBMCloudProvider._NAME
    """IBM Cloud Provider short name"""

    ORACLE = OracleCloudProvider._NAME
    """Oracle Cloud Provider short name"""

    # Add any new cloud provider classes to this list
    _CLOUD_PROVIDER_CLASSES = [
        AmazonCloudProvider,
        AzureCloudProvider,
        GoogleCloudProvider,
        IBMCloudProvider,
        OracleCloudProvider,
        AlibabaCloudProvider,
    ]

    def __init__(self, rpms, dmidcd, yrl, rhsm_cfg, cloud_query, subman_facts):
        cp_objs = [
            cls(
                rpms=rpms,
                dmidcd=dmidcd,
                yum_repos=yrl,
                rhsm_cfg=rhsm_cfg,
                cloud_query=cloud_query,
                subman_facts=subman_facts,
            )
            for cls in self._CLOUD_PROVIDER_CLASSES
        ]
        self.cp_bios_vendor = dict((cp._NAME, cp.cp_bios_vendor) for cp in cp_objs)
        self.cp_bios_version = dict((cp._NAME, cp.cp_bios_version) for cp in cp_objs)
        self.cp_rpms = dict((cp._NAME, cp.cp_rpms) for cp in cp_objs)
        self.cp_yum = dict((cp._NAME, cp.cp_yum) for cp in cp_objs)
        self.cp_asset_tag = dict((cp._NAME, cp.cp_asset_tag) for cp in cp_objs)
        self.cp_uuid = dict((cp._NAME, cp.cp_uuid) for cp in cp_objs)
        self.cp_manufacturer = dict((cp._NAME, cp.cp_manufacturer) for cp in cp_objs)
        self.cp_rhsm_server_hostname = dict(
            (cp._NAME, cp.cp_rhsm_server_hostname) for cp in cp_objs
        )
        self.cp_cloud_query = dict((cp._NAME, cp.cp_cloud_query) for cp in cp_objs)
        self.cp_subman_facts = dict((cp._NAME, cp.cp_subman_facts) for cp in cp_objs)
        # Identify it
        self.cloud_provider = self._select_provider()
        self.long_name = None
        if self.cloud_provider:
            for CP in cp_objs:
                if self.cloud_provider == CP._NAME:
                    self.long_name = CP.long_name

    def _select_provider(self):
        """
        This method provides the logic to identify which cloud provider is present.

        If new data sources and/or cloud providers are added you must add logic here to
        identify the new cloud provider.

        Returns:
            str: Returns the name of the cloud provider, corresponds to ``name`` property
                 in cloud provider classes.  If no cloud provider is identified,
                 ``None`` is returned
        """
        # 1. Check SubscriptionManagerFacts first
        for name, result in self.cp_subman_facts.items():
            if result:
                return name
        # 2. Check BIOS vendor
        for name, result in self.cp_bios_vendor.items():
            if result:
                return name
        # 3. Specific vendor not detected, so check BIOS version
        for name, result in self.cp_bios_version.items():
            if result:
                return name
        # 4. asset_tag / yum
        if self.cp_yum[self.AZURE] or self.cp_asset_tag[self.AZURE]:
            return self.AZURE
        # 5. asset_tag & uuid
        if self.cp_uuid[self.AWS] and self.cp_asset_tag[self.AWS]:
            return self.AWS
        # 6. manufacturer
        for name, result in self.cp_manufacturer.items():
            if result:
                return name
        # 7. Check CloudInitQuery then, it works for all supported Classes
        for name, result in self.cp_cloud_query.items():
            if result:
                return name
        # 8. Check for RPMs
        for name, result in self.cp_rpms.items():
            if result:
                return name
        # 9. rhsm server hostname
        for name, result in self.cp_rhsm_server_hostname.items():
            if result:
                return name



"""
CeilometerConf - file ``/etc/ceilometer/ceilometer.conf``
=========================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ceilometer_conf)
class CeilometerConf(IniConfigFile):
    """
    The ``/etc/ceilometer/ceilometer.conf`` file is in a standard '.ini' format,
    and this parser uses the IniConfigFile base class to read this.

    Given a file containing the following test data::

        [DEFAULT]
        #
        # From ceilometer
        http_timeout = 600
        debug = False
        verbose = False
        log_dir = /var/log/ceilometer
        meter_dispatcher=database
        event_dispatcher=database
        [alarm]
        evaluation_interval = 60
        evaluation_service=ceilometer.alarm.service.SingletonAlarmService
        partition_rpc_topic=alarm_partition_coordination
        [api]
        port = 8777
        host = 192.0.2.10
        [central]
        [collector]
        udp_address = 0.0.0.0
        udp_port = 4952
        [compute]
        [coordination]
        backend_url = redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/

    Examples:
        >>> type(config)
        <class 'insights.parsers.ceilometer_conf.CeilometerConf'>
        >>> config.sections()
        ['alarm', 'api', 'central', 'collector', 'compute', 'coordination']
        >>> config.has_option('alarm', 'evaluation_interval')
        True
        >>> config.get('coordination', 'backend_url')
        'redis://:chDWmHdH8dyjsmpCWfCEpJR87@192.0.2.7:6379/'
        >>> config.getint('collector', 'udp_port')
        4952
        >>> config.getboolean('DEFAULT', 'debug')
        False
    """
    pass



from json import dumps
from uuid import uuid4

from mock.mock import Mock, patch
from pytest import mark
from pytest import raises
from requests import ConnectionError
from requests import Timeout
from requests import codes

from insights.core.plugins import make_metadata
from insights.client.connection import InsightsConnection
from insights.client.config import InsightsConfig
from insights.util.hostname import determine_hostname


def _get_canonical_facts_response(canonical_facts):
    d = make_metadata(**canonical_facts)
    d.update(hostname=determine_hostname(), ip='10.0.0.1')
    del d["type"]
    return d


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch(
    "insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch(
    "insights.client.connection.get_canonical_facts",
    return_value=_get_canonical_facts_response({"subscription_manager_id": str(uuid4())})
)
@patch(
    "insights.client.connection.InsightsConnection.post",
    **{"return_value.status_code": codes.CREATED}
)
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_canonical_facts_request(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf):
    """
    A POST requests to the check-in endpoint is issued with correct headers and
    body containing unobfuscated Canonical Facts.
    """
    config = InsightsConfig(base_url="www.example.com", obfuscate=False)

    connection = InsightsConnection(config)
    connection.checkin()

    expected_url = connection.inventory_url + "/hosts/checkin"
    expected_headers = {"Content-Type": "application/json"}
    expected_data = get_canonical_facts.return_value
    post.assert_called_once_with(
        expected_url, headers=expected_headers, data=dumps(expected_data), log_response_text=False
    )


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch(
    "insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch(
    "insights.client.connection.get_canonical_facts",
    return_value=_get_canonical_facts_response({"subscription_manager_id": str(uuid4())})
)
@patch(
    "insights.client.connection.InsightsConnection.post",
    **{"return_value.status_code": codes.CREATED}
)
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_canonical_facts_request_cleaned(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf):
    """
    A POST requests to the check-in endpoint is issued with correct headers and
    body containing obfuscateed Canonical Facts.
    """
    config = InsightsConfig(base_url="www.example.com", obfuscate=True, obfuscate_hostname=True)

    connection = InsightsConnection(config)
    connection.checkin()

    expected_url = connection.inventory_url + "/hosts/checkin"
    expected_headers = {"Content-Type": "application/json"}
    expected_data = get_canonical_facts.return_value
    expected_data = connection._clean_facts(expected_data)
    post.assert_called_once_with(
        expected_url, headers=expected_headers, data=dumps(expected_data), log_response_text=False
    )


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch(
    "insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch("insights.client.connection.generate_machine_id", return_value=str(uuid4()))
@patch("insights.client.connection.get_canonical_facts", side_effect=RuntimeError())
@patch(
    "insights.client.connection.InsightsConnection.post",
    **{"return_value.status_code": codes.CREATED}
)
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_insights_id_request(get_proxies, post, get_canonical_facts, generate_machine_id, fetch_system_by_machine_id, rm_conf):
    """
    A POST requests to the check-in endpoint is issued with correct headers and
    body containing only an Insights ID if Canonical Facts collection fails.
    """
    config = InsightsConfig(base_url="www.example.com")

    connection = InsightsConnection(config)
    connection.checkin()

    expected_url = connection.inventory_url + "/hosts/checkin"
    expected_headers = {"Content-Type": "application/json"}
    expected_data = {"insights_id": generate_machine_id.return_value}
    post.assert_called_once_with(
        expected_url, headers=expected_headers, data=dumps(expected_data), log_response_text=False
    )


@mark.parametrize(("exception",), ((ConnectionError,), (Timeout,)))
@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch(
    "insights.client.connection.get_canonical_facts",
    return_value=_get_canonical_facts_response({"subscription_manager_id": "notauuid"})
)
@patch("insights.client.connection.InsightsConnection.post")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_request_http_failure(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf, exception):
    """
    If the checkin-request fails, None is returned.
    """
    post.side_effect = exception

    config = InsightsConfig(base_url="www.example.com")

    connection = InsightsConnection(config)
    result = connection.checkin()
    assert result is None


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch("insights.client.connection.get_canonical_facts", return_value={})
@patch(
    "insights.client.connection.InsightsConnection.post",
    **{"side_effect": RuntimeError()}
)
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_request_unknown_exception(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf):
    """
    If an unknown exception occurs, the call crashes.
    """
    config = InsightsConfig(base_url="www.example.com")
    connection = InsightsConnection(config)

    expected_exception = type(post.side_effect)
    with raises(expected_exception):
        connection.checkin()


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch("insights.client.connection.get_canonical_facts", return_value={})
@patch(
    "insights.client.connection.InsightsConnection.post",
    **{"return_value.status_code": codes.CREATED}
)
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_response_success(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf):
    """
    If a CREATED status code is received, the check-in was successful.
    """
    config = InsightsConfig(base_url="www.example.com")
    connection = InsightsConnection(config)

    result = connection.checkin()
    assert result is True


@mark.parametrize(
    ("status_code",),
    ((codes.OK,), (codes.BAD_REQUEST,), (codes.NOT_FOUND,), (codes.SERVER_ERROR,))
)
@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=True)
@patch("insights.client.connection.get_canonical_facts", return_value=_get_canonical_facts_response({}))
@patch("insights.client.connection.InsightsConnection.post")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_response_failure(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf, status_code):
    """
    If an unexpected status code is received, the check-in failed and an exception is raised.
    """
    post.return_value.status_code = status_code

    config = InsightsConfig(base_url="www.example.com")
    connection = InsightsConnection(config)

    with raises(Exception):
        connection.checkin()


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.InsightsConnection._init_session", Mock())
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=False)
@patch("insights.client.connection.get_canonical_facts")
@patch("insights.client.connection.InsightsConnection.post")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_unregistered_host(get_proxies, post, get_canonical_facts, fetch_system_by_machine_id, rm_conf):
    """
    If the host is not registered, the checkin fails.
    """
    config = InsightsConfig(base_url="www.example.com")
    connection = InsightsConnection(config)

    result = connection.checkin()
    assert result is False
    get_canonical_facts.assert_not_called()
    post.assert_not_called()



"""
Custom datasource compliance
============================

Only when option `--compliance` is specified, the insights collection will
collect data for compliance.  When it is specified, only the Insights Spec
:py:attr:`insights.specs.default.DefaultSpecs.compliance` will be
collected. To collect other specs within the collection of compliance data,
add them to the "persist" section in the "compliance_manifest"
pre-defined in the :py:mod:`insights.specs.datasources.manifests`
"""

import logging
import os
import pkgutil
import sys
import yaml
import json

from tempfile import NamedTemporaryFile
from traceback import format_exc

from insights.core import ET
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.os_release import OsRelease
from insights.parsers.redhat_release import RedhatRelease
from insights.specs.datasources.compliance import (
    ComplianceClient,
    SSG_PACKAGE,
    REQUIRED_PACKAGES,
    constants,
)

logger = logging.getLogger(__name__)


@datasource(HostContext)
def compliance_enabled(broker):
    insights_config = broker.get('client_config')
    # Run only if `--compliance` option is enabled to insights-client
    if insights_config and hasattr(insights_config, 'compliance') and insights_config.compliance:
        return True
    raise SkipComponent("Collect compliance data only --compliance is specified")


@datasource(HostContext)
def compliance_policies_enabled(broker):
    insights_config = broker.get('client_config')
    # Run only if `--compliance-policies` option is enabled to insights-client
    if (
        insights_config
        and hasattr(insights_config, 'compliance_policies')
        and insights_config.compliance_policies
    ):
        return True
    raise SkipComponent("Run only when --compliance-policies is specified")


@datasource(HostContext)
def compliance_assign_enabled(broker):
    insights_config = broker.get('client_config')
    # Run only if `--compliance-assign` option is enabled to insights-client
    if (
        insights_config
        and hasattr(insights_config, 'compliance_assign')
        and insights_config.compliance_assign
    ):
        return True
    raise SkipComponent("Run only when --compliance-assign is specified")


@datasource(HostContext)
def compliance_unassign_enabled(broker):
    insights_config = broker.get('client_config')
    # Run only if `--compliance-unassign` option is enabled to insights-client
    if (
        insights_config
        and hasattr(insights_config, 'compliance_unassign')
        and insights_config.compliance_unassign
    ):
        return True
    raise SkipComponent("Run only when --compliance-unassign is specified")


@datasource(
    HostContext,
    [OsRelease, RedhatRelease],
    optional=[
        compliance_assign_enabled,
        compliance_enabled,
        compliance_policies_enabled,
        compliance_unassign_enabled,
    ]
)
def os_version(broker):
    os_release = None
    if OsRelease in broker:
        os_release = broker[OsRelease].get("VERSION_ID")
    if not os_release and RedhatRelease in broker:
        os_release = broker[RedhatRelease].version
    if os_release:
        return os_release.split('.')
    if any(com in broker for com in [compliance_assign_enabled, compliance_enabled, compliance_policies_enabled, compliance_unassign_enabled]):
        sys.exit(constants.sig_kill_bad)
    else:
        raise SkipComponent


@datasource(
    HostContext,
    InstalledRpms,
    optional=[
        compliance_assign_enabled,
        compliance_enabled,
        compliance_policies_enabled,
        compliance_unassign_enabled,
    ]
)
def package_check(broker):
    rpms = broker[InstalledRpms]
    missed = [rpm for rpm in REQUIRED_PACKAGES if rpm not in rpms]
    if missed:
        if any(com in broker for com in [compliance_assign_enabled, compliance_enabled, compliance_policies_enabled, compliance_unassign_enabled]):
            msg = 'Missing required packages for compliance scanning. Please ensure the following packages are installed: {0}\n'.format(
                ', '.join(missed)
            )
            logger.error(msg)
            sys.exit(constants.sig_kill_bad)
        else:
            raise SkipComponent

    return rpms.newest(SSG_PACKAGE).version


@datasource(compliance_enabled, os_version, package_check, HostContext, timeout=0)
def compliance(broker):
    """
    Collect compliance data when '--compliance' is specified.
    """
    try:
        insights_config = broker.get('client_config')

        compliance = ComplianceClient(
            os_version=broker[os_version], ssg_version=broker[package_check], config=insights_config
        )

        # Preparations
        policies = compliance.get_system_policies()
        if not policies:
            logger.error(
                "System is not associated with any policies. Assign policies using the Compliance web UI.\n"
            )
            sys.exit(constants.sig_kill_bad)

        results_need_repair = compliance.results_need_repair()
        compliance_result = list()
        # OSCAP scan
        for policy in policies:
            profile_ref_id = policy['ref_id']
            file_name = 'oscap_results-{0}'.format(profile_ref_id)
            results_file = NamedTemporaryFile(prefix=file_name, suffix='.xml', delete=True)
            tailoring_file = compliance.download_tailoring_file(policy)
            compliance.run_scan(
                profile_ref_id,
                compliance.find_scap_policy(profile_ref_id),
                results_file.name,
                tailoring_file_path=tailoring_file,
            )
            # TODO: align with core collection
            obfuscation_list = insights_config.obfuscation_list
            if obfuscation_list:
                tree = ET.parse(results_file.name)
                # Retrieve the list of xpaths that need to be obfuscated
                xpaths = yaml.load(
                    pkgutil.get_data('insights', 'compliance_obfuscations.yaml'),
                    Loader=yaml.SafeLoader,
                )
                if 'ipv4' in obfuscation_list or 'ipv6' in obfuscation_list:
                    # Obfuscate IP addresses in the XCCDF report
                    compliance.obfuscate(tree, xpaths['obfuscate_ip'])
                if 'hostname' in obfuscation_list:
                    # Obfuscate the hostname in the XCCDF report
                    compliance.obfuscate(tree, xpaths['obfuscate_hostname'])
                # Overwrite the results file with the obfuscations
                tree.write(results_file.name)
            if tailoring_file:
                os.remove(tailoring_file)
            # Store result as DatasourceProvider
            with open(results_file.name, 'r') as f:
                content = [l.rstrip("\n") for l in f]
                if results_need_repair:
                    content = compliance.repair_results(results_file.name, content)
                compliance_result.append(
                    DatasourceProvider(
                        content=content,  # The actual result content here
                        relative_path='{0}.xml'.format(file_name),
                    )
                )
        if compliance_result:
            return compliance_result
    except Exception as err:
        err_msg = "Unexpected exception in compliance: {0}".format(str(err))
        logger.error(err_msg)
        logger.debug(format_exc())

    logger.error("No scan results were produced.")
    sys.exit(constants.sig_kill_bad)


@datasource(compliance_policies_enabled, os_version, package_check, HostContext, timeout=0)
def compliance_policies(broker):
    """
    Run when '--compliance-policies' is specified.
    """
    try:
        insights_config = broker.get('client_config')

        compliance = ComplianceClient(
            os_version=broker[os_version], ssg_version=broker[package_check], config=insights_config
        )

        # --compliance-policies was called
        result = compliance.assignable_policies()
        sys.exit(result)

    except Exception as err:
        err_msg = "Unexpected exception in compliance: {0}".format(str(err))
        logger.error(err_msg)
        logger.debug(format_exc())
        sys.exit(constants.sig_kill_bad)


@datasource(compliance_assign_enabled, os_version, package_check, HostContext, timeout=0)
def compliance_assign(broker):
    """
    Run when '--compliance-assign ID' is specified.
    """
    try:
        insights_config = broker.get('client_config')

        compliance = ComplianceClient(
            os_version=broker[os_version], ssg_version=broker[package_check], config=insights_config
        )

        # --compliance-assign was called
        result = compliance.policy_link(insights_config.compliance_assign, 'patch')
        sys.exit(result)

    except Exception as err:
        err_msg = "Unexpected exception in compliance: {0}".format(str(err))
        logger.error(err_msg)
        logger.debug(format_exc())
        sys.exit(constants.sig_kill_bad)


@datasource(compliance_unassign_enabled, os_version, package_check, HostContext, timeout=0)
def compliance_unassign(broker):
    """
    Run when '--compliance-unassign ID' is specified.
    """
    try:
        insights_config = broker.get('client_config')

        compliance = ComplianceClient(
            os_version=broker[os_version], ssg_version=broker[package_check], config=insights_config
        )

        # --compliance-unassign was called
        result = compliance.policy_link(insights_config.compliance_unassign, 'delete')
        sys.exit(result)

    except Exception as err:
        err_msg = "Unexpected exception in compliance: {0}".format(str(err))
        logger.error(err_msg)
        logger.debug(format_exc())
        sys.exit(constants.sig_kill_bad)


@datasource(os_version, package_check, HostContext, timeout=0)
def compliance_advisor_rule_enabled(broker):
    try:
        result = {}
        insights_config = broker.get('client_config')
        compliance = ComplianceClient(
            os_version=broker[os_version], ssg_version=broker[package_check], config=insights_config
        )
        policies = compliance.get_system_policies()
        if not policies:
            raise SkipComponent
        result['enabled_policies'] = policies
        tailoring_policies = []
        for policy in policies:
            tailoring_content = compliance.fetch_tailoring_content(policy)
            if tailoring_content:
                tailoring_policy = dict(ref_id=policy['ref_id'])
                xml_root = ET.fromstring(tailoring_content)
                pre_tag = xml_root.tag.split("Tailoring")[0]
                profile_select_tag = pre_tag + 'Profile/' + pre_tag + 'select'
                profile_select_info = xml_root.findall(profile_select_tag)
                tailoring_policy['check_items'] = [item.attrib for item in profile_select_info]
                tailoring_policies.append(tailoring_policy)
        if tailoring_policies:
            result['tailoring_policies'] = tailoring_policies
        return DatasourceProvider(
            content=json.dumps(result),
            relative_path='insights_datasources/compliance_enabled_policies'
        )

    except Exception:
        raise SkipComponent



"""
LoginPamConf - file ``/etc/pam.d/login``
========================================
"""

from insights import parser
from insights.parsers.pam import PamDConf
from insights.specs import Specs


@parser(Specs.login_pam_conf)
class LoginPamConf(PamDConf):
    """
    Parse the `/etc/pam.d/login` PAM configuration file.

    See the :py:class:`insights.parsers.pam.PamDConf` class documentation for
    more information on how this class is used.

    Sample PAM configuration for login service::

        #%PAM-1.0
        auth [user_unknown=ignore success=ok ignore=ignore default=bad] pam_securetty.so
        auth       substack     system-auth
        auth       include      postlogin
        account    required     pam_nologin.so
        account    include      system-auth
        password   include      system-auth
        # pam_selinux.so close should be the first session rule
        session    required     pam_selinux.so close
        session    required     pam_loginuid.so
        session    optional     pam_console.so

    Examples:
        >>> login_pam_conf[0].interface
        'auth'
        >>> login_pam_conf[0].control_flags[0]
        ControlFlag(flag='user_unknown', value='ignore')
        >>> login_pam_conf[0].module_name
        'pam_securetty.so'
        >>> login_pam_conf[6].module_args
        'close'
    """
    pass



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import authselect
from insights.parsers.authselect import AuthSelectCurrent
from insights.tests import context_wrap

AUTHSELECT_CURRENT_1 = """
Profile ID: sssd
Enabled features:
- with-sudo
- with-mkhomedir
- with-smartcard
""".strip()

AUTHSELECT_CURRENT_2 = """
Profile ID: custom/password-policy
Enabled features: None
""".strip()

AUTHSELECT_CURRENT_EMPTY = ""

AUTHSELECT_CURRENT_NG = """
No existing configuration detected.
""".strip()


def test_authselect_current():
    asc = AuthSelectCurrent(context_wrap(AUTHSELECT_CURRENT_1))
    assert asc.profile_id == 'sssd'
    assert asc.enabled_features == ['with-sudo', 'with-mkhomedir', 'with-smartcard']

    asc = AuthSelectCurrent(context_wrap(AUTHSELECT_CURRENT_2))
    assert asc.profile_id == 'custom/password-policy'
    assert asc.enabled_features == []


def test_authselect_current_exp():
    with pytest.raises(SkipComponent):
        AuthSelectCurrent(context_wrap(AUTHSELECT_CURRENT_EMPTY))

    with pytest.raises(SkipComponent):
        AuthSelectCurrent(context_wrap(AUTHSELECT_CURRENT_NG))


def test_doc_examples():
    env = {
        'asc': AuthSelectCurrent(context_wrap(AUTHSELECT_CURRENT_1))
    }
    failed, total = doctest.testmod(authselect, globs=env)
    assert failed == 0



"""
Pmrep - command ``pmrep -t 1s -T 1s <metrics> -o csv``
======================================================

Parse the content of the ``pmrep -t 1s -T 1s network.interface.out.packets network.interface.collisions swap.pagesout mssql.memory_manager.stolen_server_memory mssql.memory_manager.total_server_memory -o csv`` command.

Sample ``pmrep -t 1s -T 1s network.interface.out.packets network.interface.collisions swap.pagesout -o csv`` command output::

    Time,"network.interface.out.packets-lo","network.interface.out.packets-eth0","network.interface.collisions-lo","network.interface.collisions-eth0","swap.pagesout"
    2021-04-26 05:42:24,,,,,
    2021-04-26 05:42:25,1.000,2.000,3.000,4.000,5.000

Examples:
    >>> type(pmrep_doc_obj)
    <class 'insights.parsers.pmrep.PMREPMetrics'>
    >>> pmrep_doc_obj = sorted(pmrep_doc_obj, key=lambda x: x['name'])
    >>> pmrep_doc_obj[3]
    {'name': 'network.interface.collisions-eth0', 'value': '4.000'}
    >>> pmrep_doc_obj[6]
    {'name': 'network.interface.out.packets-lo', 'value': '1.000'}
    >>> pmrep_doc_obj[7]
    {'name': 'swap.pagesout', 'value': '5.000'}
"""
from csv import DictReader

from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.parsers import keyword_search
from insights.specs import Specs


@parser(Specs.pmrep_metrics)
class PMREPMetrics(CommandParser, list):
    """Parses output of ``pmrep -t 1s -T 1s <metrics> -o csv`` command."""
    def parse_content(self, content):
        if not content or len(content) == 1:
            raise SkipComponent("There is no data in the table")
        try:
            reader = DictReader(content)
        except Exception:
            raise ParseException("The content isn't in csv format")
        for k, v in dict(list(reader)[-1]).items():
            self.append(dict(name=k, value=v))

    def search(self, **kwargs):
        """
        Get the rows by searching the table with kwargs.
        This uses the :py:func:`insights.parsers.keyword_search` function for
        searching; see its documentation for usage details. If no search
        parameters are given, no rows are returned.

        Returns:
            list: A list of dictionaries of rows that match the given
            search criteria.

        Examples:
            >>> sorted(pmrep_doc_obj_search.search(name__endswith='lo'), key=lambda x: x['name'])
            [{'name': 'network.interface.collisions-lo', 'value': '3.000'}, {'name': 'network.interface.out.packets-lo', 'value': '1.000'}]
            >>> sorted(pmrep_doc_obj_search.search(name__endswith='swap.pagesout'), key=lambda x: x['name'])
            [{'name': 'swap.pagesout', 'value': '5.000'}]
        """
        return keyword_search(self, **kwargs)



from pytest import mark

from insights.cleaner import Cleaner
from insights.client.config import InsightsConfig

test_data = 'testabc\nabcd\n \n\n1234\npwd: p4ssw0rd\ntest123\npwd:abc\n'.splitlines()


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_filters_allowlist(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, None)
    ret = pp.clean_content(test_data, allowlist={'test': 2, 'pwd': 1})
    # content IS changed
    assert test_data != ret
    assert 'testabc' in ret  # 1 of 2 matched
    assert 'test123' in ret  # 2 of 2 matched
    # lines are processed in reverse order
    assert 'pwd:abc' in ret  # 1 of 1 matched
    assert 'pwd: p4ssw0rd' not in ret  # Max count matched
    assert '1234' not in ret
    assert 'abcd' not in ret


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_filters_allowlist_empty(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, None)
    ret = pp.clean_content(test_data, allowlist={})
    # content IS changed
    assert test_data != ret
    assert ret == []


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_filters_allowlist_not(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, None)
    ret = pp.clean_content(test_data, allowlist=None)
    # content IS NOT changed
    assert test_data == ret



"""
crypto-policies - files in ``/etc/crypto-policies/back-ends/``
==============================================================

This is a collection of parsers that all deal with the generated configuration
files under the ``/etc/crypto-policies/back-ends/`` folder.  Parsers included
in this module are:

CryptoPoliciesConfig - file ``/etc/crypto-policies/config``
-----------------------------------------------------------

CryptoPoliciesStateCurrent - file ``/etc/crypto-policies/state/current``
------------------------------------------------------------------------

CryptoPoliciesOpensshserver - file ``/etc/crypto-policies/back-ends/opensshserver.config``
------------------------------------------------------------------------------------------

CryptoPoliciesBind - file ``/etc/crypto-policies/back-ends/bind.config``
------------------------------------------------------------------------
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.crypto_policies_config)
class CryptoPoliciesConfig(Parser):
    """
    This parser reads the ``/etc/crypto-policies/config`` file.
    The contents of the file is a single-line value, available
    in the ``value`` property.

    Sample Input::

        LEGACY

    Examples:
        >>> cp_c.value
        'LEGACY'
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("/etc/crypto-policies/config is empty")
        self.value = get_active_lines(content)[0]


@parser(Specs.crypto_policies_state_current)
class CryptoPoliciesStateCurrent(Parser):
    """
    This parser reads the ``/etc/crypto-policies/state/current`` file.
    The contents of the file is a single-line value, available
    in the ``value`` property.

    Sample Input::

        LEGACY

    Examples:
        >>> cp_sc.value
        'LEGACY'
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("/etc/crypto-policies/state/current is empty")
        self.value = get_active_lines(content)[0]


@parser(Specs.crypto_policies_opensshserver)
class CryptoPoliciesOpensshserver(Parser, dict):
    """
    This parser reads the ``/etc/crypto-policies/back-ends/opensshserver.config`` file.

    Sample Input on RHEL8::

        CRYPTO_POLICY='-oCiphers=aes256-gcm@openssh.com,3des-cbc -oMACs=umac-128-etm@openssh.com'

    Sample Input on RHEL9::

        Ciphers aes256-gcm@openssh.com,chacha20-poly1305@openssh.com,aes256-ctr,aes128-gcm@openssh.com,aes128-ctr
        MACs hmac-sha2-256-etm@openssh.com,hmac-sha1-etm@openssh.com,umac-128-etm@openssh.com,hmac-sha2-512-etm@openssh.com,hmac-sha2-256,hmac-sha1,umac-128@openssh.com,hmac-sha2-512

    Examples:
        >>> 'CRYPTO_POLICY' in cp_os
        True
        >>> cp_os.options
        {'Ciphers': 'aes256-gcm@openssh.com,3des-cbc', 'MACs': 'umac-128-etm@openssh.com'}
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("/etc/crypto-policies/back-ends/opensshserver.config is empty")

        result = {}
        for line in get_active_lines(content):
            if "=" in line:
                key, value = line.split('=', 1)
                result[key.strip()] = value.strip("' ")
            elif line[0].isupper():
                key, value = line.split(' ', 1)
                result[key.strip()] = value.strip()
        self.update(result)

    @property
    def options(self):
        """return the configuratios as dict format"""
        whole_configuration = self.get('CRYPTO_POLICY', None)
        if whole_configuration and whole_configuration.startswith("-o"):
            result = {}
            configurations = whole_configuration.split("-o")
            for item in configurations[1:]:
                key, value = item.split("=", 1)
                result[key.strip()] = value.strip()
            return result
        else:
            return self


@parser(Specs.crypto_policies_bind)
class CryptoPoliciesBind(Parser):
    """
    This parser reads the ``/etc/crypto-policies/back-ends/bind.config`` file.
    The sections ``disable-algorithms`` and ``disable-ds-digests`` are in the
    properties ``disable_algorithms`` and ``disable_ds_digests``.

    Sample Input::

        disable-algorithms "." {
        RSAMD5;
        DSA;
        };
        disable-ds-digests "." {
        GOST;
        };

    Examples:
        >>> 'GOST' in cp_bind.disable_ds_digests
        True
        >>> cp_bind.disable_algorithms
        ['RSAMD5', 'DSA']
    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("/etc/crypto-policies/back-ends/bind.config is empty")
        self.value = content
        in_da = False
        in_ddd = False
        da = []
        ddd = []
        for line in self.value:
            if line.strip().lower().startswith("disable-algorithms"):
                in_da = True
                continue
            if line.strip().lower().startswith("disable-ds-digests"):
                in_ddd = True
                continue
            if line.strip().startswith("}"):
                in_da = False
                in_ddd = False
                continue
            algo = line.strip().strip(''';'"''')
            if in_da:
                da.append(algo)
            if in_ddd:
                ddd.append(algo)
        self.disable_algorithms = da
        self.disable_ds_digests = ddd



from insights.client import InsightsClient
from insights.client.config import InsightsConfig
from insights.client.constants import InsightsConstants as constants
from mock.mock import Mock, patch
from pytest import fixture
from tempfile import NamedTemporaryFile


@fixture
def insights_client():
    config = InsightsConfig(http_timeout=123)
    client = InsightsClient(config)
    client.connection = Mock(**{
        "base_url": "http://www.example.com/",
        "get.return_value.headers.items.return_value": [],
        "get.return_value.status_code": 200,
        "get.return_value.content": b"",
    })
    return client


def test_request_with_etag(insights_client):
    """
    An egg fetch request with Etag is issued with correct timeout set.
    """
    etag_file = NamedTemporaryFile('w+t')
    etag_value = 'some_etag'
    etag_file.write(etag_value)
    etag_file.seek(0)

    source_path = 'some-source-path'
    insights_client._fetch(source_path, etag_file.name, "", force=False)

    url = "{0}{1}".format(insights_client.connection.base_url, source_path)
    headers = {'If-None-Match': etag_value}
    insights_client.connection.get.assert_called_once_with(url, headers=headers, log_response_text=False)


def test_request_forced(insights_client):
    """
    A forced egg fetch request is issued with correct timeout set.
    """
    source_path = 'some-source-path'
    insights_client._fetch(source_path, "", "", force=False)

    url = "{0}{1}".format(insights_client.connection.base_url, source_path)
    insights_client.connection.get.assert_called_once_with(url, log_response_text=False)


@patch('insights.client.InsightsClient._fetch', Mock())
@patch('insights.client.InsightsClient.get_egg_url', return_value='/testvalue')
@patch('insights.client.write_data_to_file')
def test_egg_release_written(write_data_to_file, get_egg_url, insights_client):
    '''
    Verify egg release file successfully written after request
    '''
    insights_client.fetch(force=False)
    write_data_to_file.assert_called_once_with('/testvalue', constants.egg_release_file)


@patch('insights.client.InsightsClient._fetch')
@patch('insights.client.InsightsClient.get_egg_url', return_value='/testvalue')
@patch('insights.client.write_data_to_file')
def test_egg_release_error(write_data_to_file, get_egg_url, _fetch, insights_client):
    '''
    Verify OSError and IOError are caught and process continues on
    '''
    write_data_to_file.side_effect = OSError('test')
    assert insights_client.fetch(force=False)
    write_data_to_file.assert_called_once_with('/testvalue', constants.egg_release_file)
    assert _fetch.call_count == 2

    write_data_to_file.side_effect = None
    write_data_to_file.reset_mock()
    _fetch.reset_mock()

    write_data_to_file.side_effect = IOError('test')
    assert insights_client.fetch(force=False)
    write_data_to_file.assert_called_once_with('/testvalue', constants.egg_release_file)
    assert _fetch.call_count == 2
